/**
 *
 * Copyright (c) 2005 Conitec Datasystems GmbH
 * Written 2005 by Peer Georgi <georgi@conitec.com>
 *
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for complete details.
 **
*/


#include "cLDBA.h"
#include <sc/sc_config.h>
#include <sc/sys_debug.h>
#include "sc_UDP.h"

//#define LDBA_DEBUG_MODE

#define LDBA_UDP_CMD_SET                  0xf121 // host -> device //
#define LDBA_UDP_CMD_GET                  0xf1a1 // device -> host //

#ifdef LDBA_DEBUG_MODE
  #define LDBA_DEBUG(s) { scDbg("LDBA-DBG: "s); }
#else
  #define LDBA_DEBUG(s)
#endif

#define LDBA_FATAL(s) { scDbg("LDBA-DBG: "s); }


extern void ledSetSingle(unsigned int iIdx);
extern void ledToggle(void);
extern void ledInit(void);


/**
  Handles requests received by udp.
*/
char ldba_handleUDPRequest(void* _pObject, AT91PS_UDP pUDP, unsigned short iReq, unsigned short wValue, unsigned short wLength)
  {
    unsigned short wCmd = wValue;
    struct cLDBA* pLDBA = (struct cLDBA*)_pObject;
    if (iReq == LDBA_UDP_CMD_SET)
      {
        if (pLDBA->miCurrentCmd)
          {
            __sc_udp_sendStall(pUDP); // busy //
            return (1);
          }

        // busy check only ??? //
        if (wCmd == LDBA_CMD_CHECK_READY) // ready check only //
          {
            __sc_udp_sendZLP(pUDP); // cmd successfully //
            return (1);
          }

        // check cmd size //
        if (wLength <= sizeof(pLDBA->mpCmdBuffer))
          {
            pLDBA->miCurrentCmdSize = wLength;
            pLDBA->miCurrentCmd = wCmd | LDBA_CMD_TRANSFER_IN_PROGRESS;
            __sc_udp_readData(pUDP, pLDBA->mpCmdBuffer, wLength);
            __sc_udp_sendZLP(pUDP);
            pLDBA->miCurrentCmd = wCmd &= ~LDBA_CMD_TRANSFER_IN_PROGRESS;
            return (1);
          }
        else
          { // too much data //
            __sc_udp_sendStall(pUDP);
            return (1);
          }
        return (1);
      }


    if (iReq == LDBA_UDP_CMD_GET)
      {
        // .. //
        // return (1);
        return (0); // not supported yet //
      }

    return (0);  // request was known and handled successfully //
  }

/**
  return value:
    1 - continue
    0 - run application at object->miCurrentStoreAddress
*/
char ldba_poll(struct cLDBA* pLDBA)
  {
    if ((!(pLDBA->miCurrentCmd & LDBA_CMD_TRANSFER_IN_PROGRESS)) && (pLDBA->miCurrentCmd))
      switch (pLDBA->miCurrentCmd)
        {
          /// Set Address Counter for future operations
          case LDBA_CMD_SET_ADDRESS:
            pLDBA->miCurrentStoreAddress = *(unsigned int*)pLDBA->mpCmdBuffer;
#ifdef LDBA_DEBUG_MODE
            LDBA_DEBUG("set address: 0x");
            scDbgPrintHex32(pLDBA->miCurrentStoreAddress);
            LDBA_DEBUG("\n");
#endif
            pLDBA->miCurrentCmd = 0;
           break;

          /// Transfer Datablock of size pLDBA->miCurrentCmdSize to pLDBA->miCurrentStoreAddress and increment store address
          case LDBA_CMD_STORE:
            {
              int i = 0;
              char* p = (char*)pLDBA->miCurrentStoreAddress;
#ifdef LDBA_DEBUG_MODE
              scDbg("  * copy 0x"); scDbgPrintHex32(pLDBA->miCurrentStoreAddress); scDbg(" (0x"); scDbgPrintHex32(pLDBA->miCurrentCmdSize); scDbg(" bytes)\n");
#endif
              for (i = 0; i < pLDBA->miCurrentCmdSize; i++)
                *p++ = pLDBA->mpCmdBuffer[i];
              pLDBA->miCurrentStoreAddress += pLDBA->miCurrentCmdSize;
              pLDBA->miCurrentCmd = 0;
              ledToggle();
             break;
            }

          /// debug for busy testing stress
          case LDBA_HANGUP:
            {
              // forget to set miCurrentCmd to zero //
              break;
            }

          /// Execute received application at pLDBA->miCurrentStoreAddress
          case LDBA_CMD_EXECUTE:
            {
              pLDBA->miCurrentCmd = 0;
              return (0); // go and execute //
            }

          /// output an unformated, zero-terminated string to debug port
          case LDBA_CMD_NOTE:
            {
              pLDBA->mpCmdBuffer[sizeof(pLDBA->mpCmdBuffer)-1] = 0; // paranoya //
              scDbg(pLDBA->mpCmdBuffer);
              pLDBA->miCurrentCmd = 0;
              break;
            }

          /// fall back
          default:
            scDbg("unknown cmd: 0x");
            scDbgPrintHex32(pLDBA->miCurrentCmd);
            scDbg("\n");
            pLDBA->miCurrentCmd = 0;
           break;

        }
    return (1);
  }

void ldba_construct(struct cLDBA* pLDBA)
  {
    pLDBA->handleUDPRequest = &ldba_handleUDPRequest;
    pLDBA->miCurrentCmd = 0;
    pLDBA->miCurrentCmdSize = 0;
    pLDBA->miCurrentStoreAddress = 0x20000000;
  }
