/**
 *
 * Copyright (c) 2005 Conitec Datasystems GmbH
 * Written 2005 by Peer Georgi <georgi@conitec.com>
 *
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for complete details.
 **
*/


#include <sc/sc.h> // system core //
#include <sc/sc_config.h>
#include <sc/sys_core.h>
#include <sc/sys_debug.h>
#include "cLDBA.h"
#include "sc_UDP.h"

extern void ledInit(void);

// Recolation:
//  0x200000   (MCU cache)

// global ldba handler //
struct cLDBA gLDBA;
extern unsigned int sc_SDRAMSize;

typedef void(*pMain)(unsigned int);


void scInitStage0()
  { }

void boot(void)
  {
    unsigned int iWaitTicks = 0;
    ledInit();
    // initialize ldba object //
    ldba_construct(&gLDBA);

    sc_sti();
    sc_waitTime(200);
//    scDbg("\n\nUSB Big Application Loader v0.9b (c) 2004,2005,2006 Conitec DataSystems GmbH.\n");
//    scDbg("Setting up USB...\n");


    {
      unsigned int* p = (unsigned int*)0x20000000;
      *p = sc_SDRAMSize;
//      pMain pp = (pMain)gLDBA.miCurrentStoreAddress;
//      pp(12);
    }

    sc_udp_init();
//    scDbg("Resetting Bus...\n");
    sc_udp_resetBUS();
    while (ldba_poll(&gLDBA) == 1)
      {
        // communicate! //
        sc_udp_poll(&gLDBA, gLDBA.handleUDPRequest);
        if (scDbgHasChar())
          {
            int i = 0;
            char b = scDbgReadChar(1);
            if (b == 13)
              {
                scDbg("\n");
                for (i = 0; i < 30; i++)
                  scDbg("-");
                scDbg("\n");
              }
            if (b == 'r')
              {
                sc_udp_resetBUS();
              }
            if (b == 'd')
              {
                __sc_udp_debug_printRegs();
              }

            if (b == 'c')
              {
                scDbg("Clear Rcv Flag\n");
                AT91C_BASE_UDP->UDP_CSR[0] &= ~(AT91C_UDP_RX_DATA_BK0);
              }

            if (b == 'w')
              {
                scDbg("Clear Wakeup interrupt flag\n");
                AT91C_BASE_UDP->UDP_ICR = (1 << 13);
              }

            if (b == 'W')
              {
                scDbg("Clear all interrupt flags\n");
                AT91C_BASE_UDP->UDP_ICR = 0xffffffff;
              }

          }
      }
    scDbg("\nShutting down...");
    iWaitTicks = sc_getTickCount();
    while (!(sc_getTickCount()-iWaitTicks > 5000))
      sc_udp_poll(&gLDBA, gLDBA.handleUDPRequest);

    scDbg("\nExecuting application at 0x"); scDbgPrintHex32(gLDBA.miCurrentStoreAddress); scDbg("\n");
//    scDbg("Press a key...");
//    while (!(scDbgHasChar()));
    sc_shutDown();
    sc_udp_done();

//    Jump(gLDBA.miCurrentStoreAddress);
    callUser(sc_SDRAMSize, (int)&scSwitchCPUClocks, gLDBA.miCurrentStoreAddress);
  }
