/**
 *
 * Copyright (c) 2005 Conitec Datasystems GmbH
 * Written 2005 by Peer Georgi <georgi@conitec.com>
 *
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for complete details.
 **
*/


#include <sc/sc.h> // system core //
#include <sc/sys_core.h>
#include <at/include/at91rm9200-def.h>
#include <libdf/spi_dataflash.h>
#include <asm/mcu-direct.h>

#include "ledSupport.inl"

/**
  RELOCATION: 0x200000   (internal sram)
  This application loads blocks from the dataflash to specific locations
  in SDRAM. It can be placed to the dataflash itself to allow an automaticaly
  startup.
  This tool expects a filetable located at FILE_TABLE_FLASH_LOCATION in dataflash.
  The first item of table take a magic without any other piece of information.
  All other items are to describe what to do with them...
  This table (of max. size 64kB) will be copied to SDRAM at address FILE_TABLE_RAM_LOCATION
  in SDRAM. This memory region must not be overwritten by any parts loaded from dataflash
  descripted in this table!
  The last item of table must be zero initiaized to indicate the end of list.
  All elements of table are little endian ones!

  After all one item may be executed.

  DataFlash location: 0x0                         (boot)
  DataFlash location: FILE_TABLE_FLASH_LOCATION   (filetable) loaded to 0x20f00000 (+15MB)
*/

#include "TinyFsDef.h"

//extern void Jump(unsigned int);
extern unsigned int sc_SDRAMSize;

struct tDFDevice gBootFlash;

void scInitStage0(void)
  { }


/**
  Return Value: 0 - nothing to start (get executable)
                ~ - jump to addr
*/
unsigned int loadMemoryBlocks()
  {
    unsigned int iTry = 0;
    unsigned int iExecReturn = 0;
    unsigned int boAlternate = 0;
    unsigned int iAlternate = 0;
    struct tDFBlockTableEntry* pBlockTable = (struct tDFBlockTableEntry*)FILE_TABLE_RAM_LOCATION;
    pBlockTable[0].mDfBTEntryFlags = 0;
    iAlternate = 0;
    while (!(pBlockTable[0].mDfBTEntryFlags == FILE_TABLE_MAGIC))
      {
        if (iTry)
          {
	        scDbg(" * Retry reading file-table!... [old value: 0x");
	        scDbgPrintHex32(pBlockTable[0].mDfBTEntryFlags); scDbg("]\a\n");
            spiDFDebug();
          }
        spiDFRead(&gBootFlash, FILE_TABLE_FLASH_LOCATION, (char*)FILE_TABLE_RAM_LOCATION, 64*1024);
        iTry++;
        if (iTry > 8)
          break;
	      if (iTry == 3)
	        {
	          scDbg(" -- Reinitializing SPI interface --\n");
            spiDFInit(60000000, 15000000);
            if (!(spiDFProbeDevice(0, &gBootFlash)))
	            scDbg(" * error reinitializing SPI interface\n");
	        }
      }

    if (pBlockTable[0].mDfBTEntryFlags == 0x12345678)
      {
        unsigned int iIdx = 1;
        while (iIdx < 2048)
          {
            struct tDFBlockTableEntry* pEntry = pBlockTable+iIdx;
            if (pEntry->msItem[0] == 0) // end of list //
              return (iExecReturn);

            if (pEntry->mDfBTEntryFlags & DF_BTENTRY_FLAG_USE)
              if (pEntry->mDfBTEntryFlags & DF_BTENTRY_FLAG_COPY)
                {
                  scDbg("L "); scDbg(pEntry->msItem); scDbg(" (");
                  if (pEntry->mDfBTEntryFlags & DF_BTENTRY_FLAG_EXEC)
                    {
                      iExecReturn = pEntry->mDfBTEntryMemStart;
                      scDbg("x");
                    }
#ifdef __ALTERNATE_SUPPORT
                  if (pEntry->mDfBTEntryFlags & DF_BTENTRY_FLAG_EXEC_ALTERNATE)
                    {
                      iAlternate = pEntry->mDfBTEntryMemStart;
                      scDbg("A");
                    }
#endif
                  scDbg(")\n");

                  char bKey = 0;
                  {
                    char* pDst = (char*)pEntry->mDfBTEntryMemStart;
                    unsigned int iBytesLeft = pEntry->mDfBTEntryDFSize;
                    unsigned int iDFStart = pEntry->mDfBTEntryDFStart;
                    unsigned int iNumBytesAtOnce = 1024*100;
                    while (iBytesLeft)
                      {
                        unsigned int iSize = iBytesLeft;
                        if (iSize > iNumBytesAtOnce)
                          iSize = iNumBytesAtOnce;
                        spiDFRead(&gBootFlash, iDFStart, pDst, iSize);
                        pDst += iSize;
                        iDFStart += iSize;
                        iBytesLeft -= iSize;
                        bootLEDShowProgress();

                        if (scDbgHasChar())
                          {
                            bKey = scDbgReadChar(0);
                            if (bKey == 'a')
                              {
                                scDbg("\nAlternate boot select.\n");
                                boAlternate = 1;
                              }
                            else
                              break;
                          }

                      }
                  }
//                  spiDFRead(&gBootFlash, pEntry->mDfBTEntryDFStart, (char*)pEntry->mDfBTEntryMemStart, pEntry->mDfBTEntryDFSize);

                  if (bKey)
                    {
                      if (bKey == 'r')
                        {
                          int i = 0;
                          char* p = (char*)0x20000000;
                          scDbg("Press \"y\" to remove the boot sector...");
                          while (!(scDbgHasChar()));
                          if (scDbgReadChar(0) == 'y')
                            {
                              for (i = 0; i < 128; i++)
                                { *p = 0; p++; }
                              spiDFWrite(&gBootFlash, (char*)0x20000000, 0, 128);
                              scDbg("\n\n\n***! boot sector removed !*** \n");
                              scDbg("Restarting target...\n");
                            }
                          scReset();
                        }
                    }
                }
            iIdx++;
          }
      }
    else
      {
        scDbg("\n@!!!@ no file table\a\a\n");
        scDbg("DFP: 0x"); scDbgPrintHex32(gBootFlash.miNumPages); scDbg("\n");
        // wrong id //
        scReset();
      }
    if (boAlternate)
      {
        if (!(iAlternate))
          scDbg("** no alternate app found **\n");
         else
          iExecReturn = iAlternate;
      }
    return (iExecReturn);
  }

void boot(void)
  {
    unsigned int i;
    unsigned int iTicks = 0;
    unsigned int iRAM = FILE_TABLE_RAM_LOCATION;
    for (i = 0; i < 2*1024*1024; i++)
        mcu_write32(iRAM+(i << 2), i+0x12345678);

    sc_sti();

    bootLEDInit();

    iTicks = sc_getTickCount();
    sc_waitTime(100);
    scDbg("Checking memory...\n");
    iRAM = FILE_TABLE_RAM_LOCATION;
    for (i = 0; i < 2*1024*1024; i++)
      if (mcu_read32(iRAM+(i << 2)) != i+0x12345678)
        {
          scDbg("Memory initialization error\a at 0x");
	        scDbgPrintHex32(i); scDbg(" ");
          scDbgPrintHex32(mcu_read32(iRAM+(i << 2))); scDbg(" ");
          scDbgPrintHex32(i+0x12345678);
          scDbg("\n");
          break;
	}
//    spiDFDebug();
    spiDFInit(60000000, 15000000);

    if (!(spiDFProbeDevice(0, &gBootFlash)))
      {
        scDbg("\nError initializing dataflash!\n");
      }
    else
      {
        unsigned int iToJump = 0;
	      while (!(iToJump))
	        iToJump = loadMemoryBlocks();

        if (iToJump)
          {
            bootLEDCleanUp();
            sc_shutDown();
            callUser(sc_SDRAMSize, (int)&scSwitchCPUClocks, iToJump);
            // Jump(iToJump);
          }
        else
          scDbg("\nNo valid file-table found in dataflash!\n");
      }
    while (1);
  }
