/**
 * Copyright (c) 2005 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#include "cEMAC.h"
#include "dm9161.h"
#include "console.h"

extern "C"
  {
    #include <sc/sc.h>
    #include <sc/sc_config.h>
  }

#define AT91C_EMAC_CODE_802_3 ((unsigned int) 0x2 << 16) // (EMAC) Write Operation
#define AT91C_EMAC_RW_R       ((unsigned int) 0x2 << 28) // (EMAC) Read Operation
#define AT91C_EMAC_RW_W       ((unsigned int) 0x1 << 28) // (EMAC) Write Operation

#define EMAC_RW_MAXTRY        (1000*1000)                // maximum r/w idle/wait cycles //

#define RBF_ADDR      0xfffffffc
#define RBF_OWNER     (1<<0)
#define RBF_WRAP      (1<<1)
#define RBF_BROADCAST (1<<31)
#define RBF_MULTICAST (1<<30)
#define RBF_UNICAST   (1<<29)
#define RBF_EXTERNAL  (1<<28)
#define RBF_UNKOWN    (1<<27)
#define RBF_SIZE      0x07ff
#define RBF_LOCAL4    (1<<26)
#define RBF_LOCAL3    (1<<25)
#define RBF_LOCAL2    (1<<24)
#define RBF_LOCAL1    (1<<23)

struct tEMACRxQueue
  {
    enum FixedSettings { ElementSize = 0x600 };
    unsigned int miAddr;
    unsigned int miFlags;
  };


#define EMAC_QUEUE_WRAP_BIT              (1 << 1)
#define EMAC_QUEUE_BUFFER_DMA_UNLOCK     (1 << 0)


cEMAC::cEMAC(unsigned int iRxQueueStart, unsigned int iRxQueueElements)
  : cAbstractNetDevice(), miRxQueueStart(iRxQueueStart), miRxQueueElements(iRxQueueElements), miLastConnectionState(csDisconnected)
  {
    mpMAC = AT91C_BASE_EMAC;
    mMACProtocolParam.setAddress(0, 0); 
    mMACProtocolParam.setAddress(1, 1);
    mMACProtocolParam.setAddress(2, 2);
    mMACProtocolParam.setAddress(3, 3);
    mMACProtocolParam.setAddress(4, 4);
    mMACProtocolParam.setAddress(5, 5);
    initRxQueue();
  }

cEMAC::~cEMAC() throw()
  {
  }

unsigned int cEMAC::queueSize(unsigned int iRxQueueElements)
  {
    return (sizeof(tEMACRxQueue)*iRxQueueElements + (iRxQueueElements*tEMACRxQueue::ElementSize));
  }

bool cEMAC::isConnected()
  {
    return (miLastConnectionState != csDisconnected);
  }

/**
  This tries to initialize the EMAC.
  The return value is true on success.
*/
bool cEMAC::initialize()
  {
    bool boResult = false;
    // setup pio //
#ifdef SC_ETHERNET_PHY_CLOCK_INTERN
    // EREFCK will be generated by a 50MHz clock produced by MCU
    *AT91C_PIOA_PDR = AT91C_PA16_EMDIO | AT91C_PA15_EMDC | AT91C_PA14_ERXER | AT91C_PA13_ERX1 | AT91C_PA12_ERX0 | AT91C_PA11_ECRS_ECRSDV | AT91C_PA10_ETX1 | AT91C_PA9_ETX0 | AT91C_PA8_ETXEN;
    *AT91C_PIOA_ASR = AT91C_PA16_EMDIO | AT91C_PA15_EMDC | AT91C_PA14_ERXER | AT91C_PA13_ERX1 | AT91C_PA12_ERX0 | AT91C_PA11_ECRS_ECRSDV | AT91C_PA10_ETX1 | AT91C_PA9_ETX0 | AT91C_PA8_ETXEN;
#else
    // EREFCK will be generated by a 50MHz externaly oscillator
    *AT91C_PIOA_PDR = AT91C_PA16_EMDIO | AT91C_PA15_EMDC | AT91C_PA14_ERXER | AT91C_PA13_ERX1 | AT91C_PA12_ERX0 | AT91C_PA11_ECRS_ECRSDV | AT91C_PA10_ETX1 | AT91C_PA9_ETX0 | AT91C_PA8_ETXEN | AT91C_PA7_ETXCK_EREFCK;
    *AT91C_PIOA_ASR = AT91C_PA16_EMDIO | AT91C_PA15_EMDC | AT91C_PA14_ERXER | AT91C_PA13_ERX1 | AT91C_PA12_ERX0 | AT91C_PA11_ECRS_ECRSDV | AT91C_PA10_ETX1 | AT91C_PA9_ETX0 | AT91C_PA8_ETXEN | AT91C_PA7_ETXCK_EREFCK;
#endif
    *AT91C_PMC_PCER = 1 << AT91C_ID_EMAC; // Enable peripheral clock //

    mpMAC->EMAC_CFG |= AT91C_EMAC_CSR; // clear statistics //
//     unsigned char pMAC1[] = { 0, 1, 2, 3, 4, 5 }; // hack //
//     setMACAddress(pMAC1, 2);

    setMACAddress(mMACProtocolParam.getAddress().getAddress(), 0);

//     unsigned char pMAC3[] = { 10, 1, 2, 3, 4, 5 }; // hack //
//     setMACAddress(pMAC3, 0);

    enableMDIO();
    do
      {
        unsigned short iMsk = 0;
        if (!(read(DM9161_MDINTR, iMsk)))
          break;
        
        iMsk &= ~(DM9161_FDX_MASK | DM9161_SPD_MASK | DM9161_LINK_MASK | DM9161_INTR_MASK);
        // mask all interrupts //
        if (!(write(DM9161_MDINTR, iMsk)))
          break;

        boResult = true;
      } while (0);
    disableMDIO();

    if (boResult)
      {
        mpMAC->EMAC_RBQP = miRxQueueStart;
        mpMAC->EMAC_RSR = 0;
        mpMAC->EMAC_CFG = (mpMAC->EMAC_CFG | AT91C_EMAC_CAF    | AT91C_EMAC_NBC | AT91C_EMAC_RMII) | AT91C_EMAC_CLK_HCLK_64; //~AT91C_EMAC_CLK;
        // the following line is wrong and overclocks the PHY! (The maximum frequency allowed is 2,5MHz; A clock divisor of 8 will produce a clock of 7.5MHz on 60MHz bus clock //
//        mpMAC->EMAC_CFG = (mpMAC->EMAC_CFG | AT91C_EMAC_CAF    | AT91C_EMAC_NBC | AT91C_EMAC_RMII) & ~AT91C_EMAC_CLK;
    //                                       RcvAllValidFrames   No Broad Cast      Reduced MII          No Clock
        mpMAC->EMAC_CTL |= AT91C_EMAC_TE | AT91C_EMAC_RE;
    //                       Enable Tx       Enable Rx
      }
    return (boResult);
  }



/**
  Resets the PHY.
  Scans all MII-addresses and reset internal state machine for  using address 
  zero in future.
*/
bool cEMAC::resetPHY()
  {
    bool boResult = false;
    for (unsigned int iPHYAddr = 0; iPHYAddr < 32; iPHYAddr++)
      {
        boResult = false;
        enableMDIO();
        unsigned int iTry = 100; // EMAC_RW_MAXTRY;
//        scDbg("Reset stage 0..."); scDbgPrintDec(iPHYAddr); scDbg("\n");
        write(DM9161_DSCSR, DM9161_SM_RST, iPHYAddr);
          {
            while (iTry--)
              {
                unsigned short i = 0;
                if (read(DM9161_DSCSR, i, 0))
                  if (i && DM9161_SM_RST)
                    {
                      scDbg("\nPHY-POST success. at: "); scDbgPrintDec(iPHYAddr); scDbg("\n");
                      boResult  = true;
                      break;
                    }
              }
           }
        disableMDIO();
        if (boResult)
          break;
      }
    return (boResult);
  }

/**
  Read the PHY ID.
*/
cEMAC::PHYType cEMAC::readPHYID()
  {
    unsigned short i1 = 0;
    unsigned short i2 = 0;
    cEMAC::PHYType iResult = phyNotConnected;

    enableMDIO();
    if (read(DM9161_PHYID1, i1))
      if (read(DM9161_PHYID2, i2))
        iResult = phyUnknown;
    disableMDIO();

    if (iResult != phyNotConnected)
      {
        if ((i1 == (DM9161_PHYID1_OUI >> 6)) && ((i2 >> 10) == (DM9161_PHYID1_OUI & DM9161_LSB_MASK)))
          iResult = phyDM9161E;
      }

    return (iResult);
  }

/**
  Enable PHY access.
*/
void cEMAC::enableMDIO()
  {
    mpMAC->EMAC_CTL |= AT91C_EMAC_MPE;  // Management port enable
  }

/**
  Disable PHY access.
*/
void cEMAC::disableMDIO()
  {
    mpMAC->EMAC_CTL &= ~AT91C_EMAC_MPE;  // Management port enable
  }

/**
  Read a PHY register.
*/
bool cEMAC::read(unsigned int iReg, unsigned short& iValue, unsigned int iPHYAddr)
  {
    mpMAC->EMAC_MAN = (AT91C_EMAC_HIGH & ~AT91C_EMAC_LOW) | (AT91C_EMAC_CODE_802_3) | (AT91C_EMAC_RW_R) | (iReg << 18) | (iPHYAddr << 23);
    unsigned int iTry = EMAC_RW_MAXTRY;
    while (!(mpMAC->EMAC_SR & AT91C_EMAC_IDLE))
      {
        if (!(iTry))
          return (false);
        iTry--;
//        barrier();
      }
    iValue = (unsigned)mpMAC->EMAC_MAN & 0xffff;
    return (true);
  }

/**
  Write to a PHY register.
*/
bool cEMAC::write(unsigned int iReg, unsigned short iValue, unsigned int iPHYAddr)
  {
    mpMAC->EMAC_MAN = (AT91C_EMAC_HIGH & ~AT91C_EMAC_LOW) | (AT91C_EMAC_CODE_802_3) | (AT91C_EMAC_RW_W) | (iReg << 18) | (iValue & 0xffff) | (iPHYAddr << 23);
    unsigned int iTry = EMAC_RW_MAXTRY;
    while (!(mpMAC->EMAC_SR & AT91C_EMAC_IDLE))
      {
        if (!(iTry))
          return (false);
        iTry--;
//        barrier();
      }
    return (true);
  }

bool cEMAC::setMACAddress(const char* pETHAddr, unsigned int iNr)
  {
    if (iNr > 3)
      iNr = 0;
    switch (iNr)
      {
      case 0:
        mpMAC->EMAC_SA1L = (pETHAddr[3] << 24) | (pETHAddr[2] << 16) | (pETHAddr[1] << 8) | (pETHAddr[0]);
        mpMAC->EMAC_SA1H = (pETHAddr[5] << 8) | (pETHAddr[4]);
       break;
      case 1:
        mpMAC->EMAC_SA2L = (pETHAddr[3] << 24) | (pETHAddr[2] << 16) | (pETHAddr[1] << 8) | (pETHAddr[0]);
        mpMAC->EMAC_SA2H = (pETHAddr[5] << 8) | (pETHAddr[4]);
       break;
      case 2:
        mpMAC->EMAC_SA3L = (pETHAddr[3] << 24) | (pETHAddr[2] << 16) | (pETHAddr[1] << 8) | (pETHAddr[0]);
        mpMAC->EMAC_SA3H = (pETHAddr[5] << 8) | (pETHAddr[4]);
       break;
      case 3:
        mpMAC->EMAC_SA4L = (pETHAddr[3] << 24) | (pETHAddr[2] << 16) | (pETHAddr[1] << 8) | (pETHAddr[0]);
        mpMAC->EMAC_SA4H = (pETHAddr[5] << 8) | (pETHAddr[4]);
       break;
      }
    return (true);
  }

cEMAC::ConnectionState cEMAC::getConnectionState()
  {
    cEMAC::ConnectionState iResult = csDisconnected;
    unsigned short st1 = 0;
    unsigned short st2 = 0;
    bool boError = true;
    enableMDIO();
    if (read(DM9161_BMSR, st1))
      if (read(DM9161_DSCSR, st2))
        boError = false;
    disableMDIO();

    if (!(boError))
      {
        if (st1 & DM9161_LINK_STATUS)
          {
            if ((st1 & DM9161_100BASE_TX_FD) && (st2 & DM9161_100FDX))
              {
                mpMAC->EMAC_CFG |= AT91C_EMAC_SPD | AT91C_EMAC_FD;
                iResult = cs100MBitFD;
              }
            else
              if ((st1 & DM9161_10BASE_T_FD) && (st2 & DM9161_10FDX))
                {
                  mpMAC->EMAC_CFG = (mpMAC->EMAC_CFG & ~(AT91C_EMAC_SPD | AT91C_EMAC_FD)) | AT91C_EMAC_FD;
                  iResult = cs10MBitFD;
                }
              else
                if ((st1 & DM9161_100BASE_T4_HD) && (st2 & DM9161_100HDX))
                  {
                    mpMAC->EMAC_CFG = (mpMAC->EMAC_CFG & ~(AT91C_EMAC_SPD | AT91C_EMAC_FD)) | AT91C_EMAC_SPD;
                    iResult = cs100MBitHD;
                  }
                else
                  if ((st1 & DM9161_10BASE_T_HD) && (st2 & DM9161_10HDX))
                    {
                      mpMAC->EMAC_CFG &= ~(AT91C_EMAC_SPD | AT91C_EMAC_FD);
                      iResult = cs10MBitHD;
                    }
          }
      }
    return (iResult);
  }


cEMAC::PollStatus cEMAC::poll()
  {
    unsigned short iRcvStatus = 0;
    bool boResult = true;

    // handle link up / down //
    // [
    ConnectionState iNew = getConnectionState();
    if (iNew != miLastConnectionState)
      {
        miLastConnectionState = iNew;
        connectionChanged(iNew);
      }
    // ]

    enableMDIO();
    if (!(read(DM9161_RECR, iRcvStatus)))
      boResult = false;
    disableMDIO();

    if (!(boResult)) return (errMDIO);  // read error //
    if (iRcvStatus)
      return (errReceive);   // receive error //

    if (miCurrentRxSegment >= miRxQueueElements)
      miCurrentRxSegment = 0;

    tEMACRxQueue* pQueue = (tEMACRxQueue*)miRxQueueStart;
    tEMACRxQueue* pCurrent = &pQueue[miCurrentRxSegment];

    if (pCurrent->miAddr & EMAC_QUEUE_BUFFER_DMA_UNLOCK)
      {
        unsigned int iSize = pCurrent->miFlags & 2047;
        unsigned int iAddress = (pCurrent->miAddr & 0xfffffffc);
        if (iSize)
          {
            handleRx((char*)iAddress, iSize, pCurrent->miFlags);
          }
        pCurrent->miFlags = 0;
        pCurrent->miAddr &= 0xfffffffe; // clear bit 0 to allow receiving future frames //
        miCurrentRxSegment++;
      }

    mpMAC->EMAC_RSR |= AT91C_EMAC_REC;
    return (ready);
  }




void cEMAC::initRxQueue()
  {
    tEMACRxQueue* pQueue = (tEMACRxQueue*)miRxQueueStart;
    unsigned int iBufferStart = miRxQueueStart + (miRxQueueElements*sizeof(tEMACRxQueue));

    unsigned int iBufferSegmentSize = tEMACRxQueue::ElementSize;
    unsigned int iBufferSegmentStart = iBufferStart;

    for (unsigned int i = 0; i < miRxQueueElements; i++)
      {
        for (unsigned int iMemIdx = 0; iMemIdx < iBufferSegmentSize; iMemIdx++)
          {
            char* p = (char*)iBufferSegmentStart;
            p[iMemIdx] = iMemIdx;
          }
        tEMACRxQueue* pCurrent = &pQueue[i];
        pCurrent->miAddr = iBufferSegmentStart; // the bits 0 & 1 are zero to indicate the dma owns the buffer and no wrap here //
        pCurrent->miFlags = 0; // nothing received yet //
        iBufferSegmentStart += iBufferSegmentSize;
      }

    pQueue[miRxQueueElements-1].miAddr |= EMAC_QUEUE_WRAP_BIT; // wrap around here //
  }


void cEMAC::connectionChanged(ConnectionState i)
  {
    scDbg("Ethnernet is now ");
    scDbg((i != csDisconnected) ? ("connected ") : ("disconnected"));
    switch (i)
      {
      case cs10MBitHD:
        scDbg("10 MBits half duplex");
       break;
      case cs10MBitFD:
        scDbg("10 MBits full duplex");
       break;
      case cs100MBitHD:
        scDbg("100 MBits half duplex");
       break;
      case cs100MBitFD:
        scDbg("100 MBits full duplex");
       break;
      default:
        ;
      }
    scDbg(".\n");
  }

void cEMAC::handleRx(char* pFrame, unsigned int iSize, unsigned int iFlags)
  {
    if (iSize < 4) return;
    dispatchReceived(pFrame, iSize-4); // remove FCS (crc) //
//     scDbg("EMAC: Data received (0x"); scDbgPrintHex32(iSize); scDbg(" bytes) from ");
//
//     for (unsigned int i = 6; i < 6+6; i++)
//       {
//         scDbgPrintHex8(pFrame[i]);
//         if (i != 11)
//           scDbg(":");
//       }
//
//     if (iFlags & (1 << 26)) scDbg(" - MAC 1 match");
//     if (iFlags & (1 << 25)) scDbg(" - MAC 2 match");
//     if (iFlags & (1 << 23)) scDbg(" - MAC 3 match");
//     if (iFlags & (1 << 22)) scDbg(" - MAC 4 match");
//     scDbg(".\n");
  }

bool cEMAC::send(char* pFrame, unsigned int iSize)
  {
    unsigned int iTry = EMAC_RW_MAXTRY;
//    scDbg("Sending... "); scDbgPrintDec(iSize); scDbg(" bytes.\n");
    while (!(mpMAC->EMAC_TSR & AT91C_EMAC_BNQ))
      {
        if (!(iTry))
          return (false);
        iTry--;
      }
    mpMAC->EMAC_TAR = (long)pFrame;
    mpMAC->EMAC_TCR = iSize;
    iTry = EMAC_RW_MAXTRY;
    while (mpMAC->EMAC_TCR & 0x7ff)
      {
        if (!(iTry))
          return (false);
        iTry--;
      }
    mpMAC->EMAC_TSR |= AT91C_EMAC_COMP;
    return (true);
  }

void cEMAC::notifyProtocolParam(const cNetAbstractProtocolParam& prm)
  {
    if (prm.getID() == mMACProtocolParam.getID())
      {
        cNetMACProtocolParam& MACPrm = (cNetMACProtocolParam&)prm;
        mMACProtocolParam = MACPrm;
      }
    setMACAddress(mMACProtocolParam.getAddress().getAddress(), 0);
    scDbg("EMAC: Using address: "); mMACProtocolParam.getAddress().debugPrint(); scDbg("\n");
  }
