/**
 * Copyright (c) 2005 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#ifndef __cEMAC_h
  #define __cEMAC_h

extern "C"
  {
    #include <at/include/at91rm9200-def.h>
  }

#include "cAbstractNetDevice.h"
#include "cNetMACProtocolParam.h"

class cEMAC : public cAbstractNetDevice
  {
  public:
    enum PollStatus { ready, errReceive, errMDIO };
    enum ConnectionState { csDisconnected, cs10MBitFD, cs10MBitHD, cs100MBitFD, cs100MBitHD };
    enum PHYType { phyNotConnected, phyDM9161E, phyDM9161A, phyUnknown };
    
  public:
    /// Queue memory consumtion: iRxQueueElements*8 + (iRxQueueElements*0x600)
    cEMAC(unsigned int iRxQueueStart, unsigned int iRxQueueElements);
    virtual ~cEMAC() throw();

    /// \brief Returns required size of queue for given amount of possible elements.
    static unsigned int queueSize(unsigned int iRxQueueElements);

    bool initialize();
    bool resetPHY();
    bool isConnected();
    
    PHYType readPHYID();

    PollStatus poll();

//  protected:
    void enableMDIO();
    void disableMDIO();
    bool read(unsigned int iReg, unsigned short& iValue, unsigned int iPHYAddr = 0);
    bool write(unsigned int iReg, unsigned short iValue, unsigned int iPHYAddr = 0);
    /// Sets the specified MAC address. Valid values are 0..3.
    bool setMACAddress(const char*, unsigned int iNr = 1);
    void initRxQueue();
    /// Ready connection state from PHY and initializes the MAC
    ConnectionState getConnectionState();

    virtual void connectionChanged(ConnectionState);
    virtual void handleRx(char* pFrame, unsigned int iSize, unsigned iFlags);
    virtual void cEMAC::notifyProtocolParam(const cNetAbstractProtocolParam&);
    bool send(char* pFrame, unsigned iSize);

  private:
    AT91PS_EMAC mpMAC;
    unsigned int miRxQueueStart;
    unsigned int miRxQueueElements;
    unsigned int miCurrentRxSegment;
    ConnectionState miLastConnectionState;
    cNetMACProtocolParam mMACProtocolParam;
  };


#endif // __cEMAC_h
