#include <string.h>
#include "cLinuxTagList.h"

#define ATAG_CORE      0x54410001
#define ATAG_MEM       0x54410002
#define ATAG_RAMDISK   0x54410004
#define ATAG_CMDLINE   0x54410009
#define ATAG_NONE      0x00000000


cLinuxTagList::cLinuxTagList()
  {
    miLBOffset = 0;
  }

cLinuxTagList::~cLinuxTagList()
  {
  }


char* cLinuxTagList::getTagList()
  {
    return (mpListBuffer);
  }

unsigned int cLinuxTagList::getTagListSize()
  {
    return (miLBOffset);
  }

/**
  Creates the start tag (ATAG_CORE) and clears the buffer.
*/
void cLinuxTagList::open()
  {
    memset(mpListBuffer, 0, sizeof(mpListBuffer));
    miLBOffset = 0;
    // ATAG_CORE //
    putUInt32((5*4) >> 2);
    putUInt32(ATAG_CORE);
    putUInt32(0);
    putUInt32(0);
    putUInt32(0);
  }

/**
  Creates the last tag (ATAG_NONE) and finishes the list.
*/
void cLinuxTagList::close()
  {
    putUInt32(0);
    putUInt32(ATAG_NONE);
  }

/**
  Creates the system memory description tag.
*/
void cLinuxTagList::createMemTag(unsigned int iPhysMemStart, unsigned int iPhysMemSize)
  {
    putUInt32((4*4) >> 2);
    putUInt32(ATAG_MEM);
    putUInt32(iPhysMemSize);
    putUInt32(iPhysMemStart);

  }

/**
  Creates a command line tag.
*/
void cLinuxTagList::createCmdLineTag(const char* s)
  {
    putUInt32(((2*4)+(strlen(s)+1+4)) >> 2);
    putUInt32(ATAG_CMDLINE);
    putString(s);
    // align to 32 bits //
    miLBOffset <<= 2;
    miLBOffset >>= 2;
  }

/**
  Creates a ramdisk tag. (untested yet)
*/
void cLinuxTagList::createRamDiskTag(bool boLoad, unsigned int iStart, unsigned int iCompressedSize)
  {
    putUInt32((5*4)>>2);
    putUInt32(ATAG_RAMDISK);
    putUInt32((boLoad)?(1):(0));
    putUInt32(iCompressedSize);
    putUInt32(iStart);
  }
  
/**
  Little endian.
*/  
void cLinuxTagList::putUInt32(unsigned int i)
  {
    if (sizeof(mpListBuffer)-miLBOffset < 4) return;
    mpListBuffer[miLBOffset++] = (i & 255);
    mpListBuffer[miLBOffset++] = (i >> 8) & 255;
    mpListBuffer[miLBOffset++] = (i >> 16) & 255;
    mpListBuffer[miLBOffset++] = i >> 24;
  }

void cLinuxTagList::putString(const char* s)
  {
    if (sizeof(mpListBuffer)-miLBOffset < (strlen(s)+1)) return;
    while (*s)
      {
        mpListBuffer[miLBOffset++] = *s;
        s++;
      }
    mpListBuffer[miLBOffset++] = 0;
  }
