/**
 * Copyright (c) 2005 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#ifndef __cNetAbstractProtocol_h
  #define __cNetAbstractProtocol_h

class cAbstractNetDevice;
class cNetProtocolMgr;
class cNetAbstractProtocolParam;
class cNetAbstractPEParam;
#include "c_tSLList.h"

/**
  Protocol
    - SubProtocol1
      - SubSubProtocol
    - SubProtocol2
      - SubSubProtocol
        - SubSubSubProtocol
      - SubSubProtocol
*/

class cNetAbstractProtocol
  {
  friend class cAbstractNetDevice; // depcr
  friend class cNetProtocolMgr;
  public:
    enum RxHandleState { rxhNotHandled, rxhNotSupported, rxhHandled };
  public:
    /// This is to register a sub-protocol
    cNetAbstractProtocol(cNetAbstractProtocol&);
    /// This is to register a protocol-family
    cNetAbstractProtocol(cNetProtocolMgr&);
    /// Removes all sub-protocols and unregisters this protocol on parent
    virtual ~cNetAbstractProtocol() throw();



  protected:
    // << to implement >>
    virtual cNetAbstractProtocol::RxHandleState handleFrame(char*, unsigned int, char* pFullFrame = 0, unsigned int iFullSize = 0);
    virtual void notifyProtocolParam(const cNetAbstractProtocolParam&);
    virtual const cNetAbstractProtocolParam* hasProtocolParam(unsigned int iID);
    virtual void encapsulatePacket(char* pPacket, unsigned int iSize, const cNetAbstractPEParam* param);
    
  protected:
    /// This method will by called to indicate a new frame was received
    cNetAbstractProtocol::RxHandleState dispatchFrameToSubProtocol(char*, unsigned int, char*, unsigned int);
    /// This method pushes the packet to the [sub]-layer above
    void pushPacket(char* pPacket, unsigned int iSize, const cNetAbstractPEParam* param);
  
    /// Get associated network device.
    cAbstractNetDevice& getDevice() const;
    /// Get associated manager.
    cNetProtocolMgr& getMgr() const;
  
  private:
    /// Register a sub-protocol (used in constructor)
    void registerSubProtocol(cNetAbstractProtocol&);
    /// This is to dispatch a new protocol param to all sub-layer protocols
    void dispatchProtocolParamRecursive(const cNetAbstractProtocolParam&);
    const cNetAbstractProtocolParam* findProtocolParamRecursive(unsigned int iID);

  private:
    /// This member may be accessed by cAbstractNetDevice
    cNetAbstractProtocol* mpNextProtocol;
    /// This is to link an object of this class to the linked list of cNetProtocolMgr
    c_cSLList<cNetAbstractProtocol>::cNode mNetProtocolMgrNode;
    /// This node is to manage sub-protocols
    c_cSLList<cNetAbstractProtocol>::cNode mNetSubProtocolNode;
    /// The list node of sub protocols
    c_cSLList<cNetAbstractProtocol> mLstSubProtocols;

    /// If this pointer is nonzero the parent is the protocolmgr
    cNetProtocolMgr* mpMgrParent;
    /// If this pointer is nonzero the parent is a protocol layer
    cNetAbstractProtocol* mpProtocolParent;
  };


#endif // __cNetAbstractProtocol_h
