/**
 * Copyright (c) 2005 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#include "cNetMACAddress.h"
#include "console.h"
extern "C"
  {
    #include <sc/sc.h>
  }

cNetMACAddress::cNetMACAddress()
  {
    for (unsigned int i = 0; i < 6; i++)
      msMACAddress[i] = 0;
  }

cNetMACAddress::cNetMACAddress(char* s, unsigned int iSize)
  {
    load(s, iSize);
  }

cNetMACAddress::~cNetMACAddress() throw()
  { }

const char* cNetMACAddress::getAddress() const
  {
    return (msMACAddress);
  }

unsigned int cNetMACAddress::getSize() const
  {
    return (6);
  }

void cNetMACAddress::store(char* p) const
  {
    for (unsigned int i = 0; i < 6; i++)
      p[i] = msMACAddress[i];
  }

char* cNetMACAddress::load(char* p, unsigned int iSize)
  {
    for (unsigned int i = 0; i < 6; i++)
      {
        if (iSize > i)
          msMACAddress[i] = p[i];
        else
          msMACAddress[i] = 0;
      }
    return (msMACAddress);
  }

void cNetMACAddress::debugPrint() const
  {
    for (unsigned int i = 0; i < 6; i++)
      {
        scDbgPrintHex8(msMACAddress[i]);
        if (i != 5)
          scDbg(":");
      }
  }

void cNetMACAddress::setOctet(unsigned int iIdx, char bValue)
  {
    if (iIdx < 6)
      msMACAddress[iIdx] = bValue;
  }

char cNetMACAddress::getOctet(unsigned int iIdx) const
  {
    if (iIdx > 5)
      return (0);
    return (msMACAddress[iIdx]);
  }

bool cNetMACAddress::operator==(const cNetMACAddress& with) const
  {
    for (unsigned int i = 0; i < 6; i++)
      if (with.msMACAddress[i] != msMACAddress[i])
        return (false);
    return (true);
  }

bool cNetMACAddress::operator!=(const cNetMACAddress& with) const
  {
    return (!((*this) == with));
  }

/**
  On error the current mac address will not be changed.
*/
bool cNetMACAddress::readFromString(const char* pBuffer, unsigned int iBfSize)
  {
    unsigned char pMAC[6];
    const char* pBf = pBuffer;
    unsigned int iBytesLeft = iBfSize;
    for (unsigned int iIdx = 0; iIdx < 6; iIdx++)
      {
        NumberConversionStatus ncs;
        unsigned int iValue = getHexValue8(pBuffer+(iBfSize-iBytesLeft), iBytesLeft, ncs);
        if (ncs != NCSSuccess)
          {
            // error //
            return (false);
          }
        pMAC[iIdx] = iValue;

        if (iIdx < 5) // next character must be an ":" //
          {
            if (!(iBytesLeft))
              {
                // buffer too small //
                return (false);
              }
            if (*(pBf+(iBfSize-iBytesLeft)) != ':')
              {
                // wrong separator //
                return (false);
              }
            iBytesLeft--;
          }
      }
    for (unsigned int i = 0; i < 6; i++)
      msMACAddress[i] = pMAC[i];
    return (true);
  }

/**
  On error 0 will be returned.
  The variable iBufferBytesLeft will be checked and modified to avoid buffer overflows.
*/
unsigned int cNetMACAddress::getHexValue8(const char* pBuffer, unsigned int& iBufferBytesLeft, NumberConversionStatus& iNCS)
  {
    char bValidChars[] = "0123456789abcdefABCDEF";
    unsigned int iValue = 0;
    for (unsigned int i = 0; i < 2; i++)
      {
        iValue <<= 4;
        if (!(iBufferBytesLeft))
          {
            iNCS = NCSOutOfBuffer;
            return (0);
          }
        char b = pBuffer[i];
        bool boFound = false;
        for (int iIdx = 0; iIdx < 22; iIdx++)
          {
            if (b == bValidChars[iIdx])
              {
                if (iIdx > 15)
                  iIdx -= 6;
                iValue |= iIdx;
                boFound = true;
                iBufferBytesLeft--;
                break;
              }
            if (((b == 0) || (b == ':') || (b == ' ')) && (i != 0))
              {
                boFound = true;
                iValue >>= 4;
                break;
              }
          }
        if (!(boFound))
          {
            iNCS = NCSInvalidCharacter;
            return (0);
          }
      }
    iNCS = NCSSuccess;
    return (iValue);
  }
