/**
 * Copyright (c) 2005 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#include "cNetProtoARPCache.h"
#include "console.h"


cNetProtoARPCache::cNetProtoARPCache()
  {
    miCurrent = 0;
  }

cNetProtoARPCache::~cNetProtoARPCache() throw()
  {
  }

void cNetProtoARPCache::addCoherence(cNetIPv4Address& ipAddress, cNetMACAddress& macAddress)
  {
    for (unsigned int i = 0; i < cacheSize; i++)
      if (mCache[i].match(macAddress))
        {
          scDbg("ARP: Updating cache entry for "); ipAddress.debugPrint(); scDbg(" to "); macAddress.debugPrint(); scDbg("\n");
          mCache[i].mIPv4Address = ipAddress;
          return;
        }
    if (miCurrent >= cacheSize)
      miCurrent = 0;
    mCache[miCurrent].mIPv4Address = ipAddress;
    mCache[miCurrent].mMACAddress = macAddress;
    scDbg("ARP: Adding cache entry for "); ipAddress.debugPrint(); scDbg(" to "); macAddress.debugPrint(); scDbg("\n");
    miCurrent++;
  }

const cNetMACAddress* cNetProtoARPCache::searchMACAddress(const cNetIPv4Address& ipAddr) const
  {
    for (unsigned int i = 0; i < cacheSize; i++)
      if (mCache[i].match(ipAddr))
        return (&mCache[i].mMACAddress);
    return (0);
  }
