/**
 * Copyright (c) 2005 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#include "cNetProtoICMP.h"
#include "cNetIPv4ProtocolPEParam.h"
#include "cAbstractNetDevice.h"
#include "console.h"

extern "C"
  {
    #include <sc/sc.h>
  }


cNetProtoICMP::cNetProtoICMP(cNetAbstractProtocol& parent)
  : cNetAbstractProtocol(parent), mProtocolParam()
    {
//      scDbg("Registering protocol \"ICMP\"\n");
    }

cNetProtoICMP::~cNetProtoICMP() throw()
  {
  }

void cNetProtoICMP::notifyProtocolParam(const cNetAbstractProtocolParam& prm)
  {
    if (prm.getID() == mProtocolParam.getID())
      {
        cNetIPv4ProtocolParam& IPPrm = (cNetIPv4ProtocolParam&)prm;
        mProtocolParam = IPPrm;
      }
  }


const cNetAbstractProtocolParam* cNetProtoICMP::hasProtocolParam(unsigned int iID)
  {
    if (iID == mProtocolParam.getID())
      return (&mProtocolParam);
    return (0);
  }

struct tICMPRequestIPv4
  {
    char pSourceIP[4];      // this will passed thru the ip layer
    char pDestIP[4];        // this will passed thru the ip layer
    char bType;             // 8 echo ping
    char bCode;             // 0
    unsigned short iICMPCheckSum;
    unsigned short iICMPId;
    unsigned short iICMPSequence;
    char pData[2048];
  } __attribute__((packed));


cNetAbstractProtocol::RxHandleState cNetProtoICMP::handleFrame(char* pFrame, unsigned int iSize, char* pFullFrame, unsigned int iFullFrameSize)
  {
    if (iSize < 16) return (rxhNotHandled);
    tICMPRequestIPv4* pICMPRq = (tICMPRequestIPv4*)pFrame;
    if (pICMPRq->bType == 8) // ping
      {
        if (pICMPRq->bCode != 0)
          {
            scDbg("ICMP: unknown ping type!");
            return (rxhNotSupported);
          }
        cNetIPv4Address addrFrom;
        addrFrom.load(pICMPRq->pSourceIP);
        pICMPRq->bType = 0; // reply //
        pICMPRq->bCode = 0;
        cNetIPv4ProtocolPEParam prm(addrFrom, 0x01);
        pushPacket(pFrame, iSize, &prm);
        scDbg("ICMP: Ping request from "); addrFrom.debugPrint(); scDbg("...\n");
        return (rxhHandled);
      }
    scDbg("ICMP: frame received "); scDbgPrintDec(iSize); scDbg(" bytes.\n");
/*    cNetIPv4Address addrTarget;
    addrTarget.load(pIPv4Frame->pDestIP);
    if (addrTarget == mProtocolParam.getAddress())
      {
        scDbg("Thats me!\n");
      }
    if ((pIPv4Frame->bFrameOffset) || (pIPv4Frame->bFlags & 2))
      {
        scDbg("IP: Multible fragments are not supported yet (sorry)\n");
        return (cNetAbstractProtocol::rxhNotSupported);
      }
    // TODO: Broadcast detection (http://www.erg.abdn.ac.uk/users/gorry/course/inet-pages/ip-processing.html)
    char* pFrameToSub = pFrame+14;
    unsigned int iSubSize = iSize-14;
    return (dispatchFrameToSubProtocol(pFrameToSub, iSubSize, pFullFrame, iFullFrameSize));*/
    return (rxhNotSupported);
  }
