/**
 * Copyright (c) 2005 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#include "cNetProtoMAC.h"
#include "cAbstractNetDevice.h"
#include "cNetMACProtocolParam.h"
#include "cNetMACProtocolPEParam.h"
#include "cNetProtocolID.h"
#include "console.h"
#include <tinylibc/string.h>

extern "C"
  {
    #include <sc/sc.h>
  }


cNetProtoMAC::cNetProtoMAC(cNetProtocolMgr& mgr)
  : cNetAbstractProtocol(mgr), mMACProtocolParam()
    {
//      scDbg("Registering protocol \"MAC\"\n");
    }

cNetProtoMAC::~cNetProtoMAC() throw()
  {
  }

void cNetProtoMAC::notifyProtocolParam(const cNetAbstractProtocolParam& prm)
  {
    if (prm.getID() == mMACProtocolParam.getID())
      {
        cNetMACProtocolParam& MACPrm = (cNetMACProtocolParam&)prm;
        mMACProtocolParam = MACPrm;
      }
//    scDbg("MAC: Using address: "); mMACProtocolParam.getAddress().debugPrint(); scDbg("\n");
  }


const cNetAbstractProtocolParam* cNetProtoMAC::hasProtocolParam(unsigned int iID)
  {
    if (iID == mMACProtocolParam.getID())
      return (&mMACProtocolParam);
    return (0);
  }

struct tMACPacket
  {
    char pDest[6];
    char pFrom[6];
    char pData[0x600-12];
  };


/// The Frame Check Sequence (4 bytes) is allready removed.
cNetAbstractProtocol::RxHandleState cNetProtoMAC::handleFrame(char* pFrame, unsigned int iSize, char*, unsigned int)
  {
    tMACPacket* pMACPacket = (tMACPacket*)pFrame;
    if (iSize < 12) return (rxhNotHandled); // source+dest
    // compare dest. mac address //
    bool boIsBroadCast = true;
    bool boMatch = true;
    for (unsigned int i = 0; i < 6; i++)
      {
        if (pMACPacket->pDest[i] != 255)
          boIsBroadCast = false;
        if (pMACPacket->pDest[i] != mMACProtocolParam.getAddress().getOctet(i))
          boMatch = false;
        if ((!(boMatch)) && (!(boIsBroadCast)))
          break;
      }
    if ((boMatch) || (boIsBroadCast))
      {
         scDbg("MAC received "); scDbgPrintDec(iSize); scDbg(" bytes.");
         if (boMatch) scDbg(" (match)");
         if (boIsBroadCast) scDbg(" (broadcast)");
         scDbg("\n");
        dispatchFrameToSubProtocol(pFrame+12, iSize-12, pFrame, iSize);
      }
    return (cNetAbstractProtocol::rxhHandled);
  }

void cNetProtoMAC::encapsulatePacket(char* pPacket, unsigned int iSize, const cNetAbstractPEParam* pParam)
  {
    if (!(pParam)) return;
    if (pParam->getID() != NET_PROTOCOL_ID_MAC) return;
    // verify the param type //
    cNetMACProtocolPEParam* pMACPrm = (cNetMACProtocolPEParam*)pParam;
    
    tMACPacket macPacket;
    // fixup the destination address //
    pMACPrm->getMACAddress().store(macPacket.pDest);
   // fixup the source address //
    mMACProtocolParam.getAddress().store(macPacket.pFrom);
    // copy packet //
    if (iSize <= 0x600-12)
      memcpy(macPacket.pData, pPacket, iSize);
    // send packet //
    pushPacket((char*)&macPacket, iSize+12, 0); // add the source and dest; make place for FCS
  }
