/**
 * Copyright (c) 2005 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#include "cAbstractNetDevice.h"
#include "cNetProtocolMgr.h"
extern "C"
  {
    #include<sc/sc.h>
  }

cNetProtocolMgr::cNetProtocolMgr(cAbstractNetDevice& device)
  : mNetDevice(device)
  {
    mNetDevice.linkProtocolMgr(this);
  }

cNetProtocolMgr::~cNetProtocolMgr() throw()
  {
  }

void cNetProtocolMgr::registerProtocol(cNetAbstractProtocol& p)
  {
    mLstProtocols.append(&p.mNetProtocolMgrNode);
  }

/**
  Return the network device associated.
*/
cAbstractNetDevice& cNetProtocolMgr::getDevice() const
  {
    return (mNetDevice);
  }

/**
  A new received frame will dispatched here to all registered protocols.
*/
cNetAbstractProtocol::RxHandleState cNetProtocolMgr::dispatchFrame(char* p, unsigned int i)
  {
    cNetAbstractProtocol::RxHandleState iResult = cNetAbstractProtocol::rxhNotSupported;
    for (c_cSLList<cNetAbstractProtocol>::iterator aIt = mLstProtocols.begin(); aIt != mLstProtocols.end(); ++aIt)
      {
        iResult = (*aIt)->handleFrame(p, i);
        if (iResult != cNetAbstractProtocol::rxhNotHandled)
          break;
      }
    return (iResult);
  }


void cNetProtocolMgr::setProtocolParam(const cNetAbstractProtocolParam& prm)
  {
    mNetDevice.notifyProtocolParam(prm);
    for (c_cSLList<cNetAbstractProtocol>::iterator aIt = mLstProtocols.begin(); aIt != mLstProtocols.end(); ++aIt)
      (*aIt)->dispatchProtocolParamRecursive(prm);
  }


const cNetAbstractProtocolParam* cNetProtocolMgr::getProtocolParam(unsigned int iID)
  {
    const cNetAbstractProtocolParam* p = 0;
    for (c_cSLList<cNetAbstractProtocol>::iterator aIt = mLstProtocols.begin(); aIt != mLstProtocols.end(); ++aIt)
      {
        p = (*aIt)->findProtocolParamRecursive(iID);
        if (p) return (p);
      }
    return (0);
  }
