/**
 * Copyright (c) 2005 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#ifndef __cNetProtocolMgr_h
  #define __cNetProtocolMgr_h

/// Template for a linked list
#include "c_tSLList.h"
#include "cNetAbstractProtocol.h"
class cNetAbstractProtocolParam;
class cAbstractNetDevice;

class cNetProtocolMgr
  {
  friend class cNetAbstractProtocol;
  public:
    cNetProtocolMgr(cAbstractNetDevice&);
    ~cNetProtocolMgr() throw();

    /// This notifies all registered [sub]protocols for a new param (given data must be copied)
    void setProtocolParam(const cNetAbstractProtocolParam&);
    /// Find the first ocurence of the request parameter in all protocols
    const cNetAbstractProtocolParam* getProtocolParam(unsigned int iID);

    /// Get a reference to the net device
    cAbstractNetDevice& getDevice() const;

    /// This method will be called by network device
    cNetAbstractProtocol::RxHandleState dispatchFrame(char* pFrame, unsigned int iSize);

  private: // visible for cNetAbstractProtocol
    /// Link a new protocol family
    void registerProtocol(cNetAbstractProtocol&);

  private:
    c_cSLList<cNetAbstractProtocol> mLstProtocols;
    /// the network device
    cAbstractNetDevice& mNetDevice;
  };


#endif // __cNetProtocolMgr_h
