/**
 * Copyright (c) 2005 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


extern "C"
  {
    #include <sc/sc.h> // system core //
    #include <sc/sc_config.h>
    #include <at/include/at91rm9200-def.h>
  }

#include "cEMAC.h"
#include "cNetProtocolMgr.h"
#include "cNetProtoMAC.h"
#include "cNetProtoARP.h"
#include "cNetProtoIPv4.h"
#include "cNetProtoICMP.h"
#include "cNetIPv4ProtocolParam.h"
#include "console.h"
#include "cLinuxTagList.h"
#include <string.h>
/*#include "cNetProtoARP.h"
#include "cNetProtoICMP.h"*/

extern "C" void Jump(unsigned int);
extern "C" void startKernel(void);
extern "C" void boot(unsigned int, unsigned int);

/**
  Memory Layout:
    0x20780000-0x207e0000:     This application (max. 458752 bytes)
    0x207ff000-0x207ff0ff:     MAC-Address (256 bytes)
    0x207ff100-0x207ff2ff:     Append-Line (256 bytes) (beginning with "append:")
    0x20004000-0x20007fff:     space for page tables
    0x20008000-0x20008000:     linux kernel
**/

#define CONFIG_SETUP_MAC
#define CONFIG_NO_INTERACTIVE

#define LDLINUX_CONFIG_AREA_LOCATION           0x207ff000
#define LDLINUX_CONFIG_AREA_SIZE               0x100
#define LDLINUX_APPEND_AREA_LOCATION           0x207ff100
#define LDLINUX_APPEND_AREA_SIZE               0x200

enum FixedSettings { STACK_SIZE = 1024*1024, EMAC_QUEUE_LENGTH = 10 };

// conitec mac address range: 00:11:ef:xx:yy:zz //
#define LDLINUX_CONFIG_DEFAULT_MAC_ADDRESS     "00:11:ef:00:00:00"

extern "C" void scInitStage0();

void scInitSage0()
  { }

void appendCmdLine(char* sDst, const char* s, unsigned int& iPosition, unsigned iSize)
  {
    int iLeft = iSize-iPosition-1;
    if (iPosition && iLeft && (*s))
      {
        sDst[iPosition++] = 32;
        iLeft--;
      }
        
    while (iLeft)
      {
        if (*s)
          sDst[iPosition++] = *s;
        else
          break;
        s++;
        iLeft--;
      }
  }

/**
  \brief This function creates the kernel tagged list.
  (Linux command line)
  sAppendLine must be zero terminated.
*/
void createLinuxCommandLine(unsigned int iMemSize, const char* sAppendLine)
  {
    iMemSize /= 1024*1024;
    char sCmdLine[4096];
    unsigned int iPos = 0;
    memset(sCmdLine, 0, 4096);
    switch (iMemSize)
      {
        case 32:
          appendCmdLine(sCmdLine, "mem=32M", iPos, 4096);
         break;
        case 64:
          appendCmdLine(sCmdLine, "mem=64M", iPos, 4096);
         break;
        case 128:
          appendCmdLine(sCmdLine, "mem=128M", iPos, 4096);
         break;
      }
    appendCmdLine(sCmdLine, sAppendLine, iPos, 4096);
    scDbg("Linux-CommandLine: "); scDbg(sCmdLine); scDbg(".\n");
    cLinuxTagList tagList;
    tagList.open();
//    tagList.createMemTag(0x20800000, iMemSize);
    tagList.createCmdLineTag(sCmdLine);
    tagList.close();
    memcpy((void*)0x20000100, tagList.getTagList(), tagList.getTagListSize());
  }




void boot(unsigned int iMemSize, unsigned int)
  {
    unsigned int iStackAddr = 0;
    sc_sti();
    scDbg("\nLinux Loader++ v1.4a (c) 2004,2005,2006 Conitec Datasystems GmbH.\n");

    // reset USB //
    AT91C_BASE_PIOD->PIO_OER = AT91C_PIO_PD9;
    AT91C_BASE_PIOD->PIO_CODR = AT91C_PIO_PD9;
    sc_waitTime(200);
    AT91C_BASE_PIOD->PIO_SODR = AT91C_PIO_PD9;
      
    unsigned int iEMACBuffer = 0x20000000+(iMemSize - STACK_SIZE - (cEMAC::queueSize(EMAC_QUEUE_LENGTH)));
    scDbg("Installed Memory: "); scDbgPrintDec(iMemSize / 1024 / 1024); scDbg("MB.\n");
    scDbg("Stack sgm: 0x"); scDbgPrintHex32((unsigned int)&iStackAddr); scDbg(".\n");
    scDbg("EMAC-Buffer: 0x"); scDbgPrintHex32(iEMACBuffer); scDbg(".\n");
    
    // is there a valid append-line ? //
    char* sAppendLine = (char*)LDLINUX_APPEND_AREA_LOCATION;
    bool boAppendLineValid = false;
    if (!strncmp(sAppendLine, "append:", 7))
      {
        sAppendLine += 7;
        bool boEndFound = false;
        for (unsigned int i = 0; i < LDLINUX_APPEND_AREA_SIZE; i++)
          if (sAppendLine[i] == '\n')
            {
              sAppendLine[i] = 0;
              boEndFound = true;
            }
        if (!(boEndFound))
          scDbg("** Invalid append line! (No newline at end of file!)\n");
        else
          boAppendLineValid = true;
      }

    if (!(boAppendLineValid))
      sAppendLine[0] = 0;
    createLinuxCommandLine(iMemSize, sAppendLine);
    scDbg("\n");

#ifdef CONFIG_SETUP_MAC
    scDbg("Initializing MAC...");
    cEMAC mac(iEMACBuffer, EMAC_QUEUE_LENGTH);
    if (!(mac.initialize()))
      {
        scDbg("- Error initializing ethernet mac!");
      }
    scDbg("\n");
    mac.resetPHY();

    cNetProtocolMgr mgr(mac);
    // support the mac-layer //
    cNetProtoMAC protoMAC(mgr);
    cNetProtoARP protoARP(protoMAC);
    cNetProtoIPv4 protoIP(protoARP);
    cNetProtoICMP protoICMP(protoIP);
    // we try supporting the arp protocol //
//    cNetProtoARP protocolARP(mac);
    // we try supporting icmp //
//    cNetProtoICMP protocolICMP(mac);

    cNetIPv4ProtocolParam IPv4Prm;

    // set MAC address //
    cNetMACProtocolParam macAddress;
    if (!(macAddress.setAddressFromString((const char*)LDLINUX_CONFIG_AREA_LOCATION, LDLINUX_CONFIG_AREA_SIZE)))
      {
        scDbg("Unable to set MAC address. (Invalid config area)\n");
        scDbg("Using default settings");
        macAddress.setAddressFromString(LDLINUX_CONFIG_DEFAULT_MAC_ADDRESS, 18);
      }
    
    mgr.setProtocolParam(macAddress);
    
    // set ip addresss //
    unsigned int iAddr = 0;
    iAddr |= 180; iAddr <<= 8;
    iAddr |= 1;  iAddr <<= 8;
    iAddr |= 168; iAddr <<= 8;
    iAddr |= 192;
    IPv4Prm.setAddress(cNetIPv4Address(iAddr));
    mgr.setProtocolParam(IPv4Prm);
    scDbg("Using IP address: "); cNetIPv4Address(iAddr).debugPrint(); scDbg("\n");
    scDbg("Using MAC address: "); macAddress.getAddress().debugPrint(); scDbg("\n");
  //  mac.linkProtocol(&protocolARP);
    //mac.linkProtocol(&protocolICMP);

     scDbg("PHY is ");
     switch (mac.readPHYID())
      {
      case cEMAC::phyNotConnected:
        scDbg("not connected!\n");
       break;
      case cEMAC::phyDM9161E:
        scDbg("a DM9161E.\n");
       break;
      case cEMAC::phyDM9161A:
        scDbg("a DM9161A.\n");
       break;
      case cEMAC::phyUnknown:
        scDbg("a unkown type.\n");
       break;
      }
#endif


#ifdef CONFIG_NO_INTERACTIVE
    scDbg("Shutting down...\n");
    sc_shutDown();
    scDbg("Starting kernel...\n");
    startKernel();
#endif

    while (1)
      {
#ifdef CONFIG_SETUP_MAC
        mac.poll();
#endif
        if (scDbgHasChar())
          {
            int i = 0;
            char b = scDbgReadChar(0);
            if (b == 13)
              {
                scDbg("\n");
                for (i = 0; i < 79; i++)
                  scDbg("-");
                scDbg("\n");
              }
            if (b == 's')
              {
                scDbg("Leave alone with linux...\n");
                break;
              }

#ifdef CONFIG_SETUP_MAC
            if (b == 'r')
              {
                scDbg("Resetting PHY... ");
                if (mac.resetPHY())
                  scDbg("- done");
                else
                  scDbg("- error");
                scDbg("\n");
              }
            if (b == 'i')
              {
                unsigned int iAddr = 0;
                iAddr |= 18; iAddr <<= 8;
                iAddr |= 0;  iAddr <<= 8;
                iAddr |= 168; iAddr <<= 8;
                iAddr |= 192;
                IPv4Prm.setAddress(cNetIPv4Address(iAddr));
                mgr.setProtocolParam(IPv4Prm);
              }
            if (b == 'a')
              {
                char pTest[100];
                scDbg("Tx...");
                pTest[0] = '0';
                pTest[1] = '1';
                pTest[2] = '2';
                pTest[3] = '3';
                pTest[4] = '4';
                pTest[5] = '5';
                pTest[6] = 0;
                pTest[7] = 0x0c;
                pTest[8] = 0x76;
                pTest[9] = 0x71;
                pTest[10] = 0x02;
                pTest[11] = 0x09;
                pTest[12] =         0x08;
                pTest[13] =         0x42;
                pTest[14] = '5';
                pTest[15] = '5';
                pTest[16] = '5';
                pTest[17] = '5';
                pTest[18] = '5';
                pTest[19] = '5';
                pTest[20] = '5';
                if (!(mac.send(pTest, 100)))
                  scDbg("Transmit failed.\n");
                else
                  scDbg(" - done\n");
              }
#endif        
          }
      }
    sc_shutDown();
    startKernel();
  }
