/**
 *
 * Copyright (c) 2005 Conitec Datasystems GmbH
 * Written 2005 by Peer Georgi <georgi@conitec.com>
 *
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for complete details.
 **
*/


#include <sc/sc.h> // system core //
#include <sc/sc_config.h>
#include <sc/sys_core.h>
#include "cFrontEnd.h"
extern void Jump(unsigned int);
extern unsigned int sc_SDRAMSize;

#define MEM_8_MB        (1024*1024*8)

struct cFrontEnd* pFrontEnd = 0;

void memset32(unsigned int* pTo, unsigned int i32, unsigned int iCount)
  {
    int i;
    for (i = 0; i < iCount; i++)
      pTo[i] = i32;
  }

void strncpy(void* _pTo, const void* _pFrom, unsigned int iCount)
  {
    int i;
    char* pTo = (char*)_pTo;
    const char* pFrom = (const char*)_pFrom;
    int iMax = iCount+1;
    for (i = 0; i < iMax; i++)
      {
        if (i >= iMax) break;
        if (i < iCount)
          pTo[i] = pFrom[i];
        else
          pTo[i] = 0;
      }
  }

int strncmp(const char * cs,const char * ct,unsigned int count)
{
    register signed char __res = 0;

    while (count) {
        if ((__res = *cs - *ct++) != 0 || !*cs++)
            break;
        count--;
    }

    return __res;
}

unsigned int strlen(char* s)
  {
    unsigned int iResult = 0;
    while (*s++ != 0)
      iResult++;
    return (iResult);
  }

void* memcpy(void* pDst, const void* pSrc, unsigned int iSize)
  {
    while(iSize--)
      *(char*)pDst++ = *(char*)pSrc++;
    return (0);
  }

void clockSwitch(int i)
  {
    unsigned int iPLLA = PLLA_VALUE_FAST(8,64);
    unsigned int iMCK = 80000000;
    unsigned int iMDIV = SC_MAIN_CLOCK_DIV2;
#if 1
    if (!(i))
      {
        iMCK = 58333333;
        iPLLA = PLLA_VALUE_FAST(8,70);
        iMDIV = SC_MAIN_CLOCK_DIV3;
      }
#else
    if (!(i))
      {
        iMCK = 66666666;
        iPLLA = PLLA_VALUE_FAST(8,80);
        iMDIV = SC_MAIN_CLOCK_DIV3;
      }
#endif
    sc_waitTime(2500);
    sc_cli();
    {
      unsigned int iRefreshPeriod = 64; // [milliseconds]
      unsigned int iRowBits = 13;       // 2^13 rows
      unsigned int iTR = ((iMCK / 1000)*iRefreshPeriod) / (1 << iRowBits);

      scInitDBGU(iMCK);
      scSwitchCPUClocks(iPLLA, PLLB_VALUE_USB(45,216), iMDIV, iTR / 2);
    }
    sc_sti();
    sc_waitTime(2500);
  }

/**
  0 = error
  1 = ok
*/
char executeMemoryTest(unsigned int iPattern, unsigned int iKind, unsigned int boUseSelfRefreshMode)
  {
    unsigned int* pSDRAMStartAddr = (unsigned int*)SC_SDRAM_BASE;
    unsigned int iStartPattern = iPattern; // for iKind == linear == 1
    unsigned int iAddr;
    unsigned int iAddrSub;
//    unsigned char* pByteMem = (unsigned char*)SC_SDRAM_BASE;
    unsigned int* pMem32 = 0;
    unsigned int iReadOut = 0;
    unsigned int iStep = 256*1024;

    pMem32 = pSDRAMStartAddr;
    if (iKind == 0) // pattern test //
      {
        for (iAddr = 0; iAddr < (sc_SDRAMSize / iStep); iAddr++)
          {
            pFrontEnd->submitMemoryTestPatternInit(iPattern, (iAddr*iStep));
            memset32(pMem32, iPattern, iStep >> 2);
            pMem32+=iStep >> 2;
          }
      }
    else
      { // linear test //
        for (iAddr = 0; iAddr < (sc_SDRAMSize / iStep); iAddr++)
          {
            pFrontEnd->submitMemoryTestPatternInit(iPattern, (iAddr*iStep));

            for (iAddrSub = 0; iAddrSub <  (iStep >> 2); iAddrSub++)
              *pMem32++ = iPattern++;
          }
      }
    pMem32 = pSDRAMStartAddr;

/*  SIMULIERTER FEHLER:
    pByteMem += ((1024*1024*31)-1);
    *pByteMem = 0;
*/
    if (boUseSelfRefreshMode)
      scSDRAMSelfRefresh();
    else
      clockSwitch(1);


    pMem32 = pSDRAMStartAddr;
    iPattern = iStartPattern;
    if (iKind == 0)
      pFrontEnd->submitMemoryTestPatternInit(iPattern, 0xffffffff);
    else
      pFrontEnd->submitMemoryTestPatternInit(iPattern, 0xfffffffe);

    if (iKind == 0)
      { // constant pattern test //
        for (iAddr = 0; iAddr < (sc_SDRAMSize / iStep); iAddr++)
          {
            pFrontEnd->submitMemoryTestProgress(iAddr*iStep, 0, 0);
            for (iAddrSub = 0; iAddrSub < (iStep >> 2); iAddrSub++)
              {
                iReadOut = (*pMem32++);
                if (iReadOut != iPattern)
                  {
                    pFrontEnd->submitMemoryTestProgress((iAddr*iStep)+(iAddrSub<<2), 1, iReadOut);
    //                scDbgReadChar(0);
                    return (0);
                  }
              }
          }
      }
    else
      { // linear test //
        for (iAddr = 0; iAddr < (sc_SDRAMSize / iStep); iAddr++)
          {
            pFrontEnd->submitMemoryTestProgress(iAddr*iStep, 0, 0);
            for (iAddrSub = 0; iAddrSub < (iStep >> 2); iAddrSub++)
              {
                iReadOut = (*pMem32++);
                if (iReadOut != iPattern)
                  {
                    pFrontEnd->submitMemoryTestPatternInit(iPattern, 0xffffffff);
                    pFrontEnd->submitMemoryTestProgress((iAddr*iStep)+(iAddrSub<<2), 1, iReadOut);
//                    scDbgReadChar(0);
                    return (0);
                  }
                iPattern++;
              }
          }
      }
    clockSwitch(0);
    return (1);
  }


/*
  0 = error
  1 = ok
*/
char handleCmd(unsigned int iCmd)
  {
    switch (iCmd)
      {
        case FE_CMD_EXECUTE_MEMORY_TEST:
          return (executeMemoryTest(pFrontEnd->miRegs[FE_REG_MEMTEST_PATTERN], pFrontEnd->miRegs[FE_REG_MEMTEST_KIND], 0));
         break;
        case FE_CMD_EXECUTE_MEMORY_TEST_SR:
          return (executeMemoryTest(pFrontEnd->miRegs[FE_REG_MEMTEST_PATTERN], pFrontEnd->miRegs[FE_REG_MEMTEST_KIND], 1));
         break;
      }
    return (1);
  }

void scInitStage0()
  { }

void boot(void)
  {
    // Turn on the first user led...
    sc_sti();
    sc_waitTime(200);

    pFrontEnd = getTerminalFrontEnd();
    pFrontEnd->handleCmd = handleCmd;
    pFrontEnd->init(sc_SDRAMSize);

    while (1)
      pFrontEnd->poll();
  }
