#ifndef __cPin_h
  #define __cPin_h
    
extern "C"
  {
    #include <sc/sc.h> // system core //
    #include <sc/sc_config.h>
    #include <at91rm9200-def.h>
  }
    
class cPin
  {
  public:
    cPin(AT91S_PIO* p, unsigned int i, const char* sPortName, const char* sComment = 0)
      : mpPIO(p), miBit(i), mboInvalid(false), msPortName(sPortName), msComment(sComment)
        { };
    
    void setupOutput() const
      {
        if (mboInvalid) return;
        mpPIO->PIO_PER = (1 << miBit);
        mpPIO->PIO_OER = (1 << miBit);
        mpPIO->PIO_CODR = (1 << miBit);
      }
      
    void setupInput() const
      {
        mpPIO->PIO_PER = (1 << miBit);
        mpPIO->PIO_ODR = (1 << miBit);
        mpPIO->PIO_SODR = (1 << miBit);
      }
    
    void set(bool bo = true)
      {
        if (mboInvalid) return;
        setupOutput();
        if (bo)
          mpPIO->PIO_SODR = (1 << miBit);
        else
          clr();
      }
    
    void clr()
      {
        if (mboInvalid) return;
        setupOutput();
        mpPIO->PIO_CODR = (1 << miBit);
      }
    
    bool state() const
      {
        return (mpPIO->PIO_PDSR & (1 << miBit));
      }
      
    bool get() const
      {
        setupInput();
        return (mpPIO->PIO_PDSR & (1 << miBit));
      }
        
    unsigned int portIndex() const
      { return (miBit);  }
    
    const char* portName() const
      { return (msPortName); }
      
    void setComment(const char* s)
      { msComment = s; }
      
    const char* comment() const
      { return (msComment); }
      
    bool invalid() const
      { return (mboInvalid); }
      
    void invalidate()
      {
        setupInput();
        mboInvalid = true;
      }
      
    void validate()
      { mboInvalid = false; }
      
    volatile AT91S_PIO* mpPIO;
    unsigned int miBit;
    bool mboInvalid;
    const char* msPortName;
    const char* msComment;
  };
  
  
#define GPIO_PD(x)    cPin(AT91C_BASE_PIOD, x, "GPIO_PD")
#define GPIO_PB(x)    cPin(AT91C_BASE_PIOB, x, "GPIO_PB")
#define GPIO_PC(x)    cPin(AT91C_BASE_PIOC, x, "GPIO_PC")
#define GPIO_PA(x)    cPin(AT91C_BASE_PIOA, x, "GPIO_PA")

#endif // __cPin_h
