/**
 *
 * Copyright (c) 2005 Conitec Datasystems GmbH
 * Written 2005 by Peer Georgi <georgi@conitec.com>
 *
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for complete details.
 **
*/


#ifndef __cCArmEvaLCD_h
  #define __cCArmEvaLCD_h

class cCArmEvaLCD
  {
  enum AccessMode { amFb, amCtrl };
  public:
    cCArmEvaLCD(char* pFrameBuffer);
    ~cCArmEvaLCD() throw();
  
    bool detect();
    
    /// bBrightness 0..100%
    void setBacklight(unsigned int bBrightness);
  
    void clearScreen();
    void setPixel(unsigned int iX, unsigned int iY, char bCol);
    void drawHLine(unsigned int iX, unsigned int iY, unsigned int iLen, char bCol = 15);
    void drawVLine(unsigned int iX, unsigned int iY, unsigned int iLen, char bCol = 15);
    void drawRect(unsigned int iX, unsigned int iY, unsigned int iW, unsigned int iH, char bCol = 15);
    
    void putChar(unsigned int iX, unsigned int iY, char iAscii, bool boTransp = false, char bCol = 15, char bBackground = 0);
    void putText(unsigned int iX, unsigned int iY, char* sText, bool boTransp = false, char bCol = 15, char bBackground = 0);
    
    void repaintAll();
    void setInverse(bool = true);
    void setMirrorX(bool = true);
    void setMirrorY(bool = true);
    
    unsigned int getFBSize() const;
    void storeScreen(char* pDest) const; // memory with at least getFBSize() bytes //
    void restoreScreen(char* pFrom);     // memory with at least getFBSize() bytes //
    
    bool drawPNM(unsigned int iX, unsigned int iY, char* pPNMAddr);
    
  private:
    void setupIO();
    void write(char);
    char read();
    char readStatus();
    void writeInstruction(char);
    void writeDisplay(char);
    char readDisplay();
    void setAccessMode(AccessMode);
    void setLCDMapping();
            
  private:
    volatile char* mpDisplay;
    char* mpFrameBuffer;
    bool mboMirrorX;
    bool mboMirrorY;
  };
    
  
#endif // cCArmEvaLCD_h
