/**
 *
 * Copyright (c) 2005 Conitec Datasystems GmbH
 * Written 2005 by Peer Georgi <georgi@conitec.com>
 *
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for complete details.
 **
*/

// dbg1: /mnt/data/server.f1/develop/conitec/carmeva.rev-3/sw/boot/app/cArmEvaLCD/
// /usr/local/carmeva/bin/compiler/gdb/build-gdb/gdb/gdb


/**
  RELOCATION: 0x21000000.
*/
extern "C"
  {
    #include <sc/sc.h> // system core //
    #include <sc/sc_config.h>
    #include <at91rm9200-def.h>
  }

#include "cCArmEvaLCD.h"
#include "console.h"

#define MEMORY_LOCATION_PNM_IMAGE     0x21100000
#define MEMORY_LOCATION_FRAME_BUFFER  0x20000000
#define MEMORY_LOCATION_PNM_BUFFER    0x21200000

extern "C" void boot();

void showHelp()
  {
    scDbg("\ncArmEvaLCD (c) 2005 Conitec Datasystems GmbH.\n\n");
    scDbg("  \"+\" - animation faster\n");
    scDbg("  \"-\" - animation slower\n");
    scDbg("  \"B\" - backlight brighter\n");
    scDbg("  \"b\" - backlight darker\n");
    scDbg("  \"i\" - inverse mode\n");
    scDbg("  \"x\" - mirror x\n");
    scDbg("  \"y\" - mirror y\n");
    scDbg("  \"p\" - toggle 5V power on / off\n");
    scDbg("\n");
  }

// from carmeva leds
enum CARMEVA_LED { led1 = 0, led2, led3, led4, lcd_bl };
typedef enum CARMEVA_LED tArmEvaLED;


#define CARMEVA_LEDS_BRIGHTNESS_MAX 64   // maximum value for pwm //

struct cCArmEVALed
  {
    tArmEvaLED miLED;         // assigned led (ledX)
    bool mboOn;               // true = on //
    // pwm //
    unsigned int miBCounter;  // brightness counter //
    unsigned int miBValue;    // brightness value //
    // soft blink //
    bool mboSoftBlink;        // true = on //
    unsigned int miSBCounter;
    unsigned int miSBTimeBase;
    unsigned int miSBMaxBrightness;
    unsigned int miSBMinBrightness;
    bool mboSBDirection;      // true -> bring up ; false bring down
  };

// prototypes //
extern "C"
  {
    void AT91F_PIO_ClearOutput(AT91PS_PIO p, unsigned int i);
    void AT91F_PIO_SetOutput(AT91PS_PIO p, unsigned int i);
    void AT91F_PIO_CfgOutput(AT91PS_PIO p, unsigned int i);
  }

struct cCArmEVALed gLEDs[4];


void carmeva_leds_init()
  {
    int i;
    AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, AT91C_PIO_PA21);
    AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, AT91C_PIO_PA25);
    AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, AT91C_PIO_PA18);
    AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, AT91C_PIO_PA26);
    AT91F_PIO_CfgOutput(AT91C_BASE_PIOA, AT91C_PIO_PA17);
    for (i = 0; i < 4; i++)
      {
        gLEDs[i].miLED = (tArmEvaLED)i;
        gLEDs[i].mboOn = true;
        gLEDs[i].miBCounter = 0;
        gLEDs[i].miBValue = 10;
        gLEDs[i].miSBCounter = 0;
        gLEDs[i].miSBTimeBase = 0;
        gLEDs[i].mboSoftBlink = false;
        gLEDs[i].mboSBDirection = true;
        gLEDs[i].miSBMinBrightness = 0;
        gLEDs[i].miSBMaxBrightness = 0;
      }
  }

void carmeva_leds_set(tArmEvaLED led, bool bo)
  {
    switch (led)
      {
        case led1:
          (bo) ? (AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, AT91C_PIO_PA21)) : (AT91F_PIO_SetOutput(AT91C_BASE_PIOA, AT91C_PIO_PA21));
         break;
        case led2:
          (bo) ? (AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, AT91C_PIO_PA25)) : (AT91F_PIO_SetOutput(AT91C_BASE_PIOA, AT91C_PIO_PA25));
         break;
        case led3:
          (bo) ? (AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, AT91C_PIO_PA26)) : (AT91F_PIO_SetOutput(AT91C_BASE_PIOA, AT91C_PIO_PA26));
         break;
        case led4:
          (bo) ? (AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, AT91C_PIO_PA18)) : (AT91F_PIO_SetOutput(AT91C_BASE_PIOA, AT91C_PIO_PA18));
         break;
        case lcd_bl:
          (bo) ? (AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, AT91C_PIO_PA17)) : (AT91F_PIO_SetOutput(AT91C_BASE_PIOA, AT91C_PIO_PA17));
         break;
      }
  }


void carmeva_leds_poll()
  {
    int i;
    for (i = 0; i < 4; i++)
      {
        struct cCArmEVALed* pLED = &gLEDs[i];
        if (!(pLED->mboOn))
          carmeva_leds_set(pLED->miLED, false);
        else
          { // let the pwm decide //
            pLED->miBCounter++;
            if (pLED->miBCounter > CARMEVA_LEDS_BRIGHTNESS_MAX)
              {
                pLED->miBCounter = 0;
                pLED->miSBCounter++;
              }
            carmeva_leds_set(pLED->miLED, pLED->miBCounter < pLED->miBValue);

            if (pLED->mboSoftBlink)
              {
                if (pLED->miSBCounter > pLED->miSBTimeBase)
                  {
                    pLED->miSBCounter = 0;
                    if (pLED->mboSBDirection)
                      {
                        if (pLED->miBValue < CARMEVA_LEDS_BRIGHTNESS_MAX)
                          pLED->miBValue++;
                      }
                    else
                      if (pLED->miBValue)
                        pLED->miBValue--;
                  }
                if ((pLED->miBValue > pLED->miSBMaxBrightness) || (pLED->miBValue <= pLED->miSBMinBrightness))
                  if (pLED->miSBMaxBrightness != pLED->miSBMinBrightness)
                    pLED->mboSBDirection ^= true;
              }
          }
      }
  }

void carmeva_leds_animate()
  {
    int i;
    for (i = 0; i < 4; i++)
      {
        gLEDs[i].miLED = (tArmEvaLED)i;
        gLEDs[i].mboOn = true;
        gLEDs[i].miBCounter = 0;
        gLEDs[i].miBValue = 10;
        gLEDs[i].miSBCounter = 0;
        gLEDs[i].miSBTimeBase = 0;
        gLEDs[i].mboSoftBlink = false;
        gLEDs[i].mboSBDirection = true;
        gLEDs[i].miSBMinBrightness = 0;
        gLEDs[i].miSBMaxBrightness = 0;
      }
    gLEDs[0].mboSoftBlink = true;
    gLEDs[0].miSBMinBrightness = 1;
    gLEDs[0].miSBMaxBrightness = 10;
    gLEDs[0].miSBTimeBase = 200;

    gLEDs[1].mboSoftBlink = true;
    gLEDs[1].miSBMinBrightness = 2;
    gLEDs[1].miSBMaxBrightness = 30;
    gLEDs[1].miSBTimeBase = 300;

    gLEDs[2].mboSoftBlink = true;
    gLEDs[2].miSBMinBrightness = 1;
    gLEDs[2].miSBMaxBrightness = 30;
    gLEDs[2].miSBTimeBase = 150;

    gLEDs[3].mboSoftBlink = true;
    gLEDs[3].miSBMinBrightness = 1;
    gLEDs[3].miSBMaxBrightness = 30;
    gLEDs[3].miSBTimeBase = 50;
  }
// ] carmeva_leds

void boot()
  {
    sc_sti();
    sc_waitTime(200);


    showHelp();

    carmeva_leds_init();
    carmeva_leds_animate();

    bool bo5VPowerOn = true;
    AT91C_BASE_PIOA->PIO_SODR = 1 << 6;
    AT91C_BASE_PIOA->PIO_OER = 1 << 6;
    AT91C_BASE_PIOA->PIO_SODR = 1 << 6;
    cCArmEvaLCD lcd((char*)MEMORY_LOCATION_FRAME_BUFFER);

    if (!(lcd.detect()))
      {
        scDbg("No LCD detected\n");
        scDbg("    - skipping !\n"); //while (1) ;
      }

    scDbg("Starting....\n");

    lcd.clearScreen();
    lcd.repaintAll();

    lcd.drawPNM(0,0, (char*)MEMORY_LOCATION_PNM_IMAGE);
    lcd.storeScreen((char*)MEMORY_LOCATION_PNM_BUFFER);

    int iX = 0;
    int iY = 0;
    int iDirX = 1;
    int iDirY = 1;
    int iWait = 1000000;
    bool boShowLogo = true;

    unsigned int iBacklight = 0;
    bool boInverse = false;
    bool boMX = false;
    bool boMY = true;

    unsigned int iRunLevel = 0;
    unsigned int iCycle = 0;
    while (1)
      {
        if (iRunLevel)
          {
            if (boShowLogo)
              {
                lcd.restoreScreen((char*)MEMORY_LOCATION_PNM_BUFFER);
                lcd.putText(iX, iY, "Hello", true);
                lcd.putText(iX, iY+16, "World", true);
                lcd.repaintAll();
                iX += iDirX;
                iY += iDirY;

                if ((iX >= 240-(8*5)) || (iX <= 0))
                  iDirX *= -1;
                if ((iY >= 160-32) || (iY <= 0))
                  iDirY *= -1;
                for (int i=0; i< iWait; i++)
                  ;;
              }
         }

        carmeva_leds_poll();
        iCycle++;
        if (iCycle > (unsigned((iRunLevel == 0) ? (10000) : (1))))
          {
            iCycle = 0;

            if (iRunLevel < 2)
              {
                lcd.setBacklight(iBacklight++);
                if (iBacklight > 20)
                  iRunLevel = 1;
                if (iBacklight == 100)
                  iRunLevel = 2;
              }
            else
              iRunLevel = 2;
          }

        if (scDbgHasChar())
          {
            int i = 0;
            char b = scDbgReadChar(1);
            if (b == 13)
              {
                scDbg("\n");
                for (i = 0; i < 30; i++)
                  scDbg("-");
                scDbg("\n");
                boShowLogo ^= true;

                if (!(boShowLogo))
                  {
                    lcd.restoreScreen((char*)MEMORY_LOCATION_PNM_BUFFER);
                    lcd.repaintAll();
                  }

              }
            if (b == 'b')
              {
                if (iBacklight > 0)
                  iBacklight--;
                lcd.setBacklight(iBacklight);
                scDbg("Setting backlight to "); scDbgPrintDec(iBacklight); scDbg("%\n");
              }
            if (b == 'B')
              {
                if (iBacklight < 100)
                  iBacklight++;
                lcd.setBacklight(iBacklight);
                scDbg("Setting backlight to "); scDbgPrintDec(iBacklight); scDbg("%\n");
              }

            if (b == 'i')
              {
                boInverse ^= true;
                lcd.setInverse(boInverse);
                scDbg("Setting iversion: "); scDbg((boInverse) ? ("on") : ("off")); scDbg("\n");
              }

            if (b == 'x')
              {
                boMX ^= true;
                lcd.setMirrorX(boMX);
                scDbg("Setting mirror x: "); scDbg((boMX) ? ("on") : ("off")); scDbg("\n");
              }

            if (b == 'y')
              {
                boMY ^= true;
                lcd.setMirrorY(boMY);
                scDbg("Setting mirror y: "); scDbg((boMY) ? ("on") : ("off")); scDbg("\n");
              }

            if (b == 'p')
              {
                bo5VPowerOn ^= true;
                scDbg("5V power: "); scDbg((bo5VPowerOn) ? ("on") : ("off")); scDbg("\n");
                AT91C_BASE_PIOA->PIO_OER = (1 << 6);
                if (bo5VPowerOn)
                  AT91C_BASE_PIOA->PIO_SODR = 1 << 6;
                else
                  AT91C_BASE_PIOA->PIO_CODR = 1 << 6;
              }
            if (b == '+')
              if (iWait > 10000)
                iWait -= 10000;

            if (b == '-')
              iWait += 10000;

            if (b == 'h')
              showHelp();
          }
      }
    while (1);
  }
