/*************************************************************************/
  .text
  .align
  .globl setup_cpu_clk, __setup_cpu_clock, __setup_cpu_slowclock

#define AT91C_BASE_PMC           0xFFFFFC00 /* (PMC) Base Address */
#define PMC_MCKR                 0x30       /* Master Clock Register */
#define PMC_SR                   0x68       /* Status Register */
#define PMC_MOR                  0x20       /* Main Oscillator Register */
#define PMC_SCER                 0x00
#define PMC_SCDR                 0x04
#define PMC_PLLAR                0x28       /* PLL A Register */
#define PMC_PLLBR                0x2C       /* PLL B Register */

#define AT91C_PMC_MOSCS       0x1 /* (PMC) MOSC Status/Enable/Disable/Mask */
#define AT91C_PMC_MCKRDY      0x8 /* (PMC) MCKRDY -//- */
#define AT91C_PMC_LOCKA       0x2 /* (PMC) PLL A Status/Enable/Disable/Mask */
#define AT91C_PMC_LOCKB       0x4 /* (PMC) PLL B Status/Enable/Disable/Mask */


#define ASYNC_BUS_MODE
//#define SYNC_BUS_MODE
// #define DISABLE_ICACHE




#include "../sc/sc_config.h"



__setup_cpu_slowclock:
/*-- Load base address of the PMC register: */
            ldr      r1, =AT91C_BASE_PMC

/*-- Change the Master Clock to Slow Clock mode */
            ldr      r0, =0x4
            str      r0, [r1, #PMC_MCKR]  /* write new value into register */
   mov pc, lr


// r1 - PLLA config
// extern "C" void __setup_cpu_clock(unsigned int iPLLAConfig, unsigned int iPLLBConfig, unsigned int iMCKDiv);
__setup_cpu_clock:
      stmfd sp!, {r0-r4}
	    mov r4, r2 // r4 = MCK-Div
      mov r3, r1 // r3 = PLLB - Config
	    mov r2, r0 // r2 = PLLA - Config
/* Set FastBus mode of the processor while changing PLLs clocks */
            ldr      r0, =0x00000078

/* b0  = 0 - MMU disable */
/* b1  = 0 - Fault checking disable */
/* b2  = 0 - DCache disable */
/* b3:6 - Reserved (1111b) */
/* b7  = 0 - Little-endian */
/* b8  = 0 - Sys protection */
/* b9  = 0 - ROM protection */
/* b10:11 - Reserved (00b) */
/* b12 = 0 - ICache disable */
/* b13 = 0 - Exception registers */
/*             at 0x00000000 */
/* b14 = 0 - Random replacement */
/* b15:29 - Reserved (000000000000000b) */
/* b30:31 = 00 - Fast Bus mode */
/* write r0 in cp15 control register (cp15 r1) */
            mcr      p15, 0, r0, c1, c0,0

/*-- Load base address of the PMC register: */
            ldr      r1, =AT91C_BASE_PMC

/*-- Change the Master Clock to Slow Clock mode */
/*---- Next step will be configuring the PLLs */
/*---- And only after that we could turn the system in full speed */
/*---- At first set dirty value (errata workaround). */

            ldr      r0, =0x4             // select PLLA
            str      r0, [r1, #PMC_MCKR]



Loop1Osc:
// Wait for Master Clock stabilization
            ldr      r0, [r1, #PMC_SR]
            tsts     r0, #AT91C_PMC_MCKRDY
            beq      Loop1Osc

            ldr      r0, =0               // select slow clock
            str      r0, [r1, #PMC_MCKR]

Loop2Osc:
// ---- Wait for Master Clock stabilization
            ldr      r0, [r1, #PMC_SR]
            tsts     r0, #AT91C_PMC_MCKRDY
            beq      Loop2Osc

/* !!!(for PLLs look in to errata too) */
/*-- Program the PLLA output freq to 0 */
            ldr      r0, =0x2000000
            str      r0, [r1, #PMC_PLLAR] /* write the value into register */

/*-- Program the PLLB output freq to 0 */
            ldr      r0, =0
            str      r0, [r1, #PMC_PLLBR] /* write the value into register */


/*---- Prepare the value to be written to enable main clock: */
/* You have to change it */
            ldr      r0, =0x0000FF01   /* bit0  = 1 - enable main oscill
                                        * byte1 = FF - time slice before
                                        *    enabling. In slow ticks/8.
                                        */

            str      r0, [r1, #PMC_MOR]  /* write the value into register */

/*
; Wait while main oscillator have started up
;-- i.e. the bit MOSCS in PMC_SR have been set
;-- it will be set when number specified in CKGR_MOR as time slice
;-- will reach the 0 value
*/


Loop3Osc:
//---- Read the value from PMC_SR:
            ldr      r0, [r1, #PMC_SR]
            tsts     r0, #AT91C_PMC_MOSCS
            beq      Loop3Osc


/*-- Program the PLLA output freq to 180MHz (for main clock = 20MHz) */
/*---- PLL output freq = (main clock)*(mul+1)/div */
//            ldr      r0, =SC_PLLA_VALUE
//            str      r0, [r1, #PMC_PLLAR] /* write the value into register */
            str      r2, [r1, #PMC_PLLAR] /* write the value into register */

/*-- Wait while PLL will pulled-up */
Loop2PLLA:
/*---- Read the value from PMC_SR: */
            ldr      r0, [r1, #PMC_SR]
            tsts     r0, #AT91C_PMC_LOCKA
            beq      Loop2PLLA

/*-- Program the PLLB output freq to 48MHz (for main clock = 20MHz) */
/*---- PLL output freq = (main clock)*(mul+1)/div */
            str      r3, [r1, #PMC_PLLBR] /* write the value into register */

/*-- Wait while PLL will pulled-up */
Loop2PLLB:
/*---- Read the value from PMC_SR: */
            ldr      r0, [r1, #PMC_SR]
            tsts     r0, #AT91C_PMC_LOCKB
            beq      Loop2PLLB

/*-- Change the Master Clock to real speed (in two steps, see errata 27,28) */

/* Second step: set real value for Master Clock in two steps: */
/* 1) Write a dirty value to PMC_MCKR */
/* 2) Write real value to PCM_MCKR */

            mov r0, r4
            bic r0, r0, #15
            str      r0, [r1, #PMC_MCKR]

            mov r0, r4
            str      r0, [r1, #PMC_MCKR]



Loop41Osc:
/*---- Wait for Master Clock stabilization: */
            ldr      r0, [r1, #PMC_SR]
            tsts     r0, #AT91C_PMC_MCKRDY
            beq      Loop41Osc

/* Write CP15 control register */
/* This will be overwriten by the Linux kernel */
#ifdef ASYNC_BUS_MODE
            ldr      r0, =0xc0005078      /* b0  = 0 - MMU disable
                                           * b1  = 1 - Fault checking disable
                                           * b2  = 0 - DCache disable
                                           * b3:6 - Reserved (1111b)
                                           * b7  = 0 - Little-endian
                                           * b8  = 0 - Sys protection
                                           * b9  = 0 - ROM protection
                                           * b10:11 - Reserved (00b)
                                           * b12 = 1 - ICache enable
                                           * b13 = 0 - Exception registers
                                           *             at 0x00000000
                                           * b14 = 1 - Round Robin replacement
                                           * b15:29 - Reserved (000000000000000b)
                                           * b30:31 = 11 - Asynchronous bus
                                           *                mode
                                           */
#ifdef DISABLE_ICACHE
            ldr      r0, =0xc0004078
#endif

            @ write r0 in cp15 control register (cp15 r1)

#else
  #ifdef SYNC_BUS_MODE
            ldr      r0, =0x40005078
  #else
// fastbus mode
            ldr      r0, =0x00001078
  #endif
#endif
            mcr      p15, 0, r0, c1, c0,0



/*-- Enable processor clock */
            ldr      r0, =0x00000001   /* bit0 = 1 - processor clock */
            str      r0, [r1, #PMC_SCER]  /* write the value into register */

/* Loop for compensate oscillator start up */


            ldr      r0, =0xFFFFFF
Loop5Osc:
            subs     r0, r0, #1
            bhi      Loop5Osc



// turning on i-cache...
//            ldr      r0, =0xc0001078
//            mcr      p15, 0, r0, c1, c0,0

      ldmfd sp!, {r0-r4}

   mov pc, lr
/* From here system is working at highest speed */
