#ifndef __spi_dataflash_h
  #define __spi_dataflash_h

/**
  \brief Includes all information about
  current command to execute.
*/
struct tDFCmdDesc
  {
    unsigned char           *tx_cmd_pt;
    unsigned int            tx_cmd_size;
    unsigned char           *rx_cmd_pt;
    unsigned int            rx_cmd_size;
    unsigned char           *tx_data_pt;
    unsigned int            tx_data_size;
    unsigned char           *rx_data_pt;
    unsigned int            rx_data_size;
    volatile unsigned char  DataFlash_state;
    unsigned char           command[8];
  };


/**
  \brief This declares a dataflash.
*/
struct tDFDevice
  {
    // << info about device >> //
    /// \brief Number of pages.
    unsigned int miNumPages;
    /// \brief Size of pages in bytes.
    unsigned int miPageSize;
    /// \brief Offset used in commands.
    unsigned int miPageOffset;
    /// \brief Mask.
    unsigned int miByteMask;
    /// \brief Assigned spi-chip-select.
    unsigned int miCS;
    /// \brief ID returned by device;
    unsigned int miID;
    /// \brief Command descriptor.
    struct tDFCmdDesc mCmd;
  };

// << prototypes >> //
extern void spiDFInit(unsigned int iMCK, unsigned int iSpeed);
extern unsigned int spiDFProbeDevice(unsigned int iCS, struct tDFDevice* pInfo);
extern unsigned int spiDFRead(struct tDFDevice* pDataFlash, unsigned int iSrc, char* pDst, unsigned int iSize);
extern unsigned int spiDFWrite(struct tDFDevice* pDataFlash, char* pSrc, unsigned int iDst, unsigned int iSize);
extern void spiDFDebug();

#endif // __spi_dataflash_h
