/**
 * Copyright (c) 2005,2006 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#include <at/include/at91rm9200-def.h>
#include <at/include/lib_at91.h>
#include <asm/mcu-direct.h>
#include <at/include/at91rm9200_sys.h>
#include <sc_config.h>
#include <sc/sys_core.h>
#include <sc/sys_debug.h>



// Beispiel (0x2188c155)
//   + Col-Bits:
// SDRAM - configuation
// SC_SDRAM_COL_BITS_9, SC_SDRAM_ROW_BITS_12, SC_SDRAM_NUM_BANKS_4, 2, 8, 1, 1, 3, 4
#define SC_SDRAM_COL_BITS_8                  0
#define SC_SDRAM_COL_BITS_9                  1
#define SC_SDRAM_COL_BITS_10                 2
#define SC_SDRAM_COL_BITS_11                 3
#define SC_SDRAM_COL_BITS(x) ((x) & 3)

#define SC_SDRAM_ROW_BITS_11                 0
#define SC_SDRAM_ROW_BITS_12                 1
#define SC_SDRAM_ROW_BITS_13                 2
#define SC_SDRAM_ROW_BITS(x) (((x) & 3) << 2)

#define SC_SDRAM_NUM_BANKS_2                 0
#define SC_SDRAM_NUM_BANKS_4                 1
#define SC_SDRAM_NUM_BANKS(x) (((x) & 1) << 4)

#define SC_SDRAM_CAS_LATENCY  (2 << 5)

// [1]2..15 //
#define SC_SDRAM_WRITE_RECOVERY_TIME(x)  (((x) & 15) << 7)

// [1]2..15 //
#define SC_SDRAM_ROW_CYCLE_DELAY(x)      (((x) & 15) << 11)

// [1]2..15 //
#define SC_SDRAM_ROW_PRECHARGE_DELAY(x)  (((x) & 15) << 15)

// [1]2..15 //
#define SC_SDRAM_ROW_TOCOLUMN_DELAY(x)   (((x) & 15) << 19)

// [1]2..15 //
#define SC_SDRAM_ACTIVE_TOPRECHARGE_DELAY(x) (((x) & 15) << 23)

// [1]2..15 //
#define SC_SDRAM_EXIT_SELF_REFRESH_DELAY(x) (((x) & 15) << 27)

//#define SC_SDRAM_SETUP(COL_BITS, ROW_BITS, NUM_BANKS, WRITE_RECOVERY_TIME, ROW_CYCLE_DELAY, ROW_PRECHARGE_DELAY, ROW_TOCOLUMN_DELAY, ACTIVE_TOPRECHARGE_DELAY, EXIT_SELF_REFRESH_DELAY)    ( SC_SDRAM_COL_BITS(COL_BITS) | SC_SDRAM_ROW_BITS(ROW_BITS) | SC_SDRAM_NUM_BANKS(NUM_BANKS) | SC_SDRAM_CAS_LATENCY | SC_SDRAM_WRITE_RECOVERY_TIME(WRITE_RECOVERY_TIME) | SC_SDRAM_ROW_CYCLE_DELAY(ROW_CYCLE_DELAY) | SC_SDRAM_ROW_PRECHARGE_DELAY(ROW_PRECHARGE_DELAY) | SC_SDRAM_ROW_TOCOLUMN_DELAY(ROW_TOCOLUMN_DELAY) | SC_SDRAM_ACTIVE_TOPRECHARGE_DELAY(ACTIVE_TOPRECHARGE_DELAY) | SC_SDRAM_EXIT_SELF_REFRESH_DELAY(EXIT_SELF_REFRESH_DELAY) )
// WITHOUT CAS LATENCY
#define SC_SDRAM_SETUP(COL_BITS, ROW_BITS, NUM_BANKS, WRITE_RECOVERY_TIME, ROW_CYCLE_DELAY, ROW_PRECHARGE_DELAY, ROW_TOCOLUMN_DELAY, ACTIVE_TOPRECHARGE_DELAY, EXIT_SELF_REFRESH_DELAY)    ( SC_SDRAM_COL_BITS(COL_BITS) | SC_SDRAM_ROW_BITS(ROW_BITS) | SC_SDRAM_NUM_BANKS(NUM_BANKS) | SC_SDRAM_WRITE_RECOVERY_TIME(WRITE_RECOVERY_TIME) | SC_SDRAM_ROW_CYCLE_DELAY(ROW_CYCLE_DELAY) | SC_SDRAM_ROW_PRECHARGE_DELAY(ROW_PRECHARGE_DELAY) | SC_SDRAM_ROW_TOCOLUMN_DELAY(ROW_TOCOLUMN_DELAY) | SC_SDRAM_ACTIVE_TOPRECHARGE_DELAY(ACTIVE_TOPRECHARGE_DELAY) | SC_SDRAM_EXIT_SELF_REFRESH_DELAY(EXIT_SELF_REFRESH_DELAY) )


// SDRAM auto detection
//   The following field contains all information the detection procedure needs to
//   detect a sdram.
//   sc_sdram_descriptor_list contains an array of tSDRAMDataSet.
struct tSDRAMDataSet
  {
    /// (Cols << 8) | (Pages+Rows)
    unsigned int miColsAndSumOfRowsAndPages;
    /// data created by SC_SDRAM_SETUP
    unsigned int miSDRAMConfig;
  };

struct tSDRAMDataSet sc_sdram_descriptor_list[] =
  {
    /// 64MB; (f.e. K4S511632x)
    {0x0a0f, SC_SDRAM_SETUP(SC_SDRAM_COL_BITS_10, SC_SDRAM_ROW_BITS_13, SC_SDRAM_NUM_BANKS_4, 2, 8, 3, 1, 1, 4)},
    /// 32-MB; (f.e. K4S561632x)
//    {0x090f, SC_SDRAM_SETUP(SC_SDRAM_COL_BITS_9, SC_SDRAM_ROW_BITS_13, SC_SDRAM_NUM_BANKS_4, 2, 8, 3, 1, 1, 4)},
    {0x090f, SC_SDRAM_SETUP(SC_SDRAM_COL_BITS_9, SC_SDRAM_ROW_BITS_13, SC_SDRAM_NUM_BANKS_4, 1, 5, 1, 0, 0, 1)},
//    {0x090f, SC_SDRAM_SETUP(SC_SDRAM_COL_BITS_9, SC_SDRAM_ROW_BITS_13, SC_SDRAM_NUM_BANKS_4, 1, 4, 0, 0, 0, 1)}, // (also works) - the 4 is critical!
    {0,0}
  };



extern int __read_fault_status(void);
extern int __read_fault_address(void);

int sc_uninitializedData;
int sc_initializedData = 0;

/*
  BEWARE!
    The variable sc_SDRAMSize must reside in "data" segment, NOT in "bss"!
    To avoid placing the variable to bss we have to initialize it with a NONZERO value!
  Background:
    The size of SDRAM will be obtained BEFORE the segments will be initialized by crt0.S .
    After the initialization sequence the variable sc_SDRAMSize keeps the size of SDRAM detected
    and crt0.S will be started.
    It copies the data-segment (initialized data) and initializes the BSS with zero.
    Because the sc_SDRAMSize resides in data section the copy loop will not influence the value. (---)
    (See crt0.S where the variable would be overwritten if it has got an nonzero initializer)
*/
unsigned int sc_SDRAMSize = 1;

extern void __sc_asm_interrupt_handler(void);

void scSetSDRAMSize(unsigned int i)
  { sc_SDRAMSize = i; }

void scHangUp(const char* s)
  {
    scDbg("\r\nHUP:");
    scDbg(s);
    scDbg("\r\n");
    while(1) { };
  }

// division by zero (backend for gcc) //
void __div0()
  {
    scHangUp("DZ");
    while(1);
  }

void sc_handleIntSpurious()
  {
    scHangUp("SI");
  }

void sc_handleIntUndef()
  {
    scHangUp("UD");
  }

void sc_handleExceptionDataAbort()
  {
    scDbgPrintChar('!');
    scDbg("DA at ");
    scDbgPrintHex32(__read_fault_address()); scDbg(" "); scDbgPrintHex32(__read_fault_status());
    scHangUp("DA");
  }

void sc_handleExceptionFetchAbort()
  {
    scHangUp("PA");
  }

void sc_handleExceptionUndef()
  {
    scDbg("\nUE!\n");
//    scHangUp("UE");
  }

/**
   \brief Writes to the specified MCU-register.
   \param i register constant (see at91rm9200_XXX register definitions)
   \param iValue 32-bit value to write
*/
void mcuWriteReg32(unsigned int i, unsigned iValue)
  { mcu_write32(i | 0xfffff000, iValue); }


/**
   \brief Reads from the specified MCU-register.
   \param i register constant (see at91rm9200_XXX register definitions)
   \return value read from register
*/
unsigned int mcuReadReg32(unsigned int i)
  { return (mcu_read32(i | 0xfffff000)); }

/**
  \brief Setup a mcu-exception handler callback.
*/
void aicSetExceptionVector(unsigned int iReg, void(*pHandler)())
  {
    unsigned int iHandler = (unsigned int)pHandler;
    unsigned int iValue = (0xe51fff20);
    if (iHandler != (0xe51fff20))
      iValue = ((((iHandler) - (iReg) - 0x8) >> 2) & 0x00ffffff) | 0xea000000;
    mcu_write32(iReg, iValue);
  }

void aicDisableInterrupt(unsigned int iIrqID)
  {
    mcuWriteReg32(AT91_AIC_IDCR, (1 << iIrqID));
    mcuWriteReg32(AT91_AIC_ICCR, (1 << iIrqID));
  }

void aicClearInterrupt(unsigned int iIrqID)
  {
    mcuWriteReg32(AT91_AIC_ICCR, (1 << iIrqID));
  }

/*
  Initializes the SDRAM controller.
  NOTE: This method needs testing! (It has been rewritten to be more cleaner!)
*/
void scInitSDRAM(unsigned int iMCK, unsigned int iSDRAMConfig, unsigned int bo32Bit)
  {
    // Configure PIOC as peripheral (D16/D31)
    if (bo32Bit)
      {
        mcuWriteReg32(AT91_PIOA | PIO_ASR, 0xFFFF0000);
        mcuWriteReg32(AT91_PIOA | PIO_PDR, 0xFFFF0000);
        mcuWriteReg32(AT91_PIOC | PIO_ASR, 0xFFFF0000);
        mcuWriteReg32(AT91_PIOC | PIO_PDR, 0xFFFF0000);
      }

    // Setup MEMC to support all connected memories (CS0 = FLASH; CS1=SDRAM)
    mcuWriteReg32(AT91_EBI_CSA, AT91_EBI_CS1A);
    mcuWriteReg32(AT91_SDRAMC_CR, iSDRAMConfig);

    // prepare "all banks prechange command" //
    if (bo32Bit)
      mcuWriteReg32(AT91_SDRAMC_MR, 0x2);
    else
      mcuWriteReg32(AT91_SDRAMC_MR, 0x02 | (1 << 4)); // 16 - bit

    // prechange all banks now //
    mcu_write32(SC_SDRAM_BASE, 0);

    // prepare a refresh command //
    if (bo32Bit)
      mcuWriteReg32(AT91_SDRAMC_MR, 0x04);
    else
      mcuWriteReg32(AT91_SDRAMC_MR, 0x04 | (1 << 4)); // 16bit

    // 8x refresh //
    mcu_write32(SC_SDRAM_BASE, 0);
    mcu_write32(SC_SDRAM_BASE, 0);
    mcu_write32(SC_SDRAM_BASE, 0);
    mcu_write32(SC_SDRAM_BASE, 0);
    mcu_write32(SC_SDRAM_BASE, 0);
    mcu_write32(SC_SDRAM_BASE, 0);
    mcu_write32(SC_SDRAM_BASE, 0);
    mcu_write32(SC_SDRAM_BASE, 0);

    // load mode register //
    if (bo32Bit)
      mcuWriteReg32(AT91_SDRAMC_MR, 0x03);
    else
      mcuWriteReg32(AT91_SDRAMC_MR, 0x03 | (1 << 4)); // 16bit

    // load value ("offset") into the SDRAM's mode register.
//    mcu_write32(SC_SDRAM_BASE+0x80, 0);
    {
      unsigned int iMR = (2 << 4) | (1 << 3) | (1 << 9);
      mcu_write32(SC_SDRAM_BASE+iMR, 0);
    }

    {
      // Refresh period is defined as time needed to refresh all rows. //
      // We have to setup the refresh timer to refresh the number of rows in
      // given refresh period time. (64ms for used SDRAMs).
      // All supported SDRAMs uses 2^13 rows -> (64ms / (2^13)) = 7.8�s. //
      // Setup refresh timer... //
      // Example: float(ev(((tRefresh/2^RowBits)/(1/MCK)=Value), MCK=60000000, tRefresh=0.064, RowBits=13));
      unsigned int iRefreshPeriod = 50; // [milliseconds]
      unsigned int iRowBits = 13;       // 2^13 rows
      unsigned int iTR = ((iMCK / 1000)*iRefreshPeriod) / (1 << iRowBits);
      mcuWriteReg32(AT91_SDRAMC_TR, iTR / 2); // example code value (eb) == iTR / 2 //
//      mcuWriteReg32(AT91_SDRAMC_TR, iTR/4); // paranoia ??  (== 0x75) //
    }
    mcu_write32(SC_SDRAM_BASE, 0);


    // setup the normal sdram mode //
    if (bo32Bit)
      mcuWriteReg32(AT91_SDRAMC_MR, 0);
    else
      mcuWriteReg32(AT91_SDRAMC_MR, 1 << 4);
    mcu_write32(SC_SDRAM_BASE, 0);
  }

void scSDRAMSelfRefresh()
  {
    // enter self refresh //
    mcuWriteReg32(AT91_SDRAMC_SRR, 1 << 0);
  }


/**
  Probes the installed system memory (SDRAM).
  NOTE
    It is not possible to detect the number of pages correctly due to a bug
    in the MCU core (see errata 32; no wrap around for SDRAM devices).
    So we find out two parameters:
      * Number of bits used for column-address
      * Sum of bits used for row-address and banks
    Theese parameters will be compared with the field "miColsAndSumOfRowsAndPages" of all
    known SDRAMs in the list "sc_sdram_descriptor_list".

  Returns
    zero - on failure
    1==1 - on success
*/
unsigned int scProbeSDRAM(unsigned int iMCK)
  {
    unsigned int iNumColumns = 0;
    unsigned int iSumOfRowsAndPages = 0;
    unsigned int i = 0;

    unsigned int iPresetColumnBits = 11;
    unsigned int iPresetRowBits = 13;
    unsigned int iPresetBankBits = 2;

#if 0
    // force 32MB //
    scInitSDRAM(60000000, SC_SDRAM_SETUP(SC_SDRAM_COL_BITS_9, SC_SDRAM_ROW_BITS_13, SC_SDRAM_NUM_BANKS_4, 2, 8, 3, 3, 3, 4), 0);
    sc_SDRAMSize = 32*1024*1024;
    return 1;
#endif

    // setup the biggest possible SDRAM type (2^11*1^13*4banks*2bytes=128MByte)
    // we are using the 16-bit mode to do that...
    scInitSDRAM(iMCK, SC_SDRAM_SETUP(SC_SDRAM_COL_BITS_11, SC_SDRAM_ROW_BITS_13, SC_SDRAM_NUM_BANKS_4, 2, 8, 3, 1, 1, 4), 0);

    // get the number of column bits //
    // the result is (NumberOfColumns-1) because we are using 32-Bit accesses vs. 16 bit sdram layout
    mcu_write32(SC_SDRAM_BASE, 0); // pMem32[0] = 0;
    for (i = 0; i < iPresetColumnBits+1; i++)
      {
        mcu_write32(SC_SDRAM_BASE + ((1 << i) << 2), i); // pMem32[(1 << i)] = i;
        if (mcu_read32(SC_SDRAM_BASE) != 0) // pMem32[0]
          {
            iNumColumns = i+1;
            break;
          }
      }

    // Find out the value "rows+banks" now and assume 4 pages //
    mcu_write32(SC_SDRAM_BASE, 0); // pMem32[0] = 0;
    for (i = 1; i < iPresetRowBits+iPresetBankBits+1; i++)
      {
        // iNumColumns-1, because 32-bit mcu vs. 16-bit memory configuation
        mcu_write32(SC_SDRAM_BASE+(((1 << i)*(1 << (iPresetColumnBits-1)) << 2)), i); // pMem32[((1 << i)*(1 << (iPresetColumnBits-1)))] = i;

        if (mcu_read32(SC_SDRAM_BASE) != 0) // pMem32[0]
          {
            iSumOfRowsAndPages = i;
            break;
          }
      }

    sc_SDRAMSize = 0;
    {
      struct tSDRAMDataSet* pDesc = sc_sdram_descriptor_list;
      while (pDesc->miColsAndSumOfRowsAndPages)
        {
          if (pDesc->miColsAndSumOfRowsAndPages == ((iNumColumns << 8) | (iSumOfRowsAndPages)))
            {
              sc_SDRAMSize = 1 << (iNumColumns+iSumOfRowsAndPages+1); // 16-Bit = 2 Bytes //

              // try setup 32-Bit mode //
              scInitSDRAM(iMCK, pDesc->miSDRAMConfig, 1);
              for (i = 0; i < 20; i++)
                {
                  mcu_write32(SC_SDRAM_BASE+(i*4), 0x12345678+i);
                }
              for (i = 0; i < 20; i++)
                {
                  if (mcu_read32(SC_SDRAM_BASE+(i*4)) != 0x12345678+i)
                    {
                                // fall back to 16-bit mode //
                      scInitSDRAM(iMCK, pDesc->miSDRAMConfig, 0);
                      return (1);
                    }
                }
              sc_SDRAMSize *= 2;
              return (1);
            }
          pDesc++;
        }
    }
    return (0);
  }

unsigned int getSDRAMSize(void)
  { return (sc_SDRAMSize); }


/*
  Initializes the Interrupt controller and exception vectors.
*/
void scInitAIC()
  {
    AT91F_AIC_Open(AT91C_BASE_AIC, AT91C_AIC_BRANCH_OPCODE, sc_handleIntUndef, sc_handleIntUndef, sc_handleIntSpurious, 0x00);

    // 8x interrupt-acknoledge
    mcuWriteReg32(AT91_AIC_EOICR, mcuReadReg32(AT91_AIC_EOICR));
    mcuWriteReg32(AT91_AIC_EOICR, mcuReadReg32(AT91_AIC_EOICR));
    mcuWriteReg32(AT91_AIC_EOICR, mcuReadReg32(AT91_AIC_EOICR));
    mcuWriteReg32(AT91_AIC_EOICR, mcuReadReg32(AT91_AIC_EOICR));
    mcuWriteReg32(AT91_AIC_EOICR, mcuReadReg32(AT91_AIC_EOICR));
    mcuWriteReg32(AT91_AIC_EOICR, mcuReadReg32(AT91_AIC_EOICR));
    mcuWriteReg32(AT91_AIC_EOICR, mcuReadReg32(AT91_AIC_EOICR));
    mcuWriteReg32(AT91_AIC_EOICR, mcuReadReg32(AT91_AIC_EOICR));

    // setup exceptions //
    aicSetExceptionVector(0x0c, sc_handleExceptionFetchAbort);
    aicSetExceptionVector(0x10, sc_handleExceptionDataAbort);
    aicSetExceptionVector(0x4, sc_handleExceptionUndef);
  }


/*
  Initialisierung des Debug-Ports.
*/
void scInitDBGU(unsigned int iMCK)
  {
   /* Setup Debug-Port */
    AT91F_US_Configure((AT91PS_USART)AT91C_BASE_DBGU, iMCK, SC_DBG_DBG_ASYNC_MODE, SC_DBG_BAUD_RATE, 0); // timeguard = 0 //
    /* Enable Transmitter */
    AT91F_US_EnableTx((AT91PS_USART)AT91C_BASE_DBGU);
    /* Enable Receiver */
    AT91F_US_EnableRx((AT91PS_USART)AT91C_BASE_DBGU);
    AT91F_DBGU_CfgPIO();
    {
      AT91PS_PMC pPMC = AT91C_BASE_PMC;
      pPMC->PMC_PCER = 0xffffffff;
    }
  }

/**
  Das Erlauben von Interrupts.
    + System-Timer (1/32768s)
*/
void scInitTimer()
  {
        // Setup SysTimer //
    AT91C_BASE_ST->ST_PIMR = 1;
    AT91F_AIC_ConfigureIt(AT91C_BASE_AIC, // AIC base address
      AT91C_ID_SYS, // System peripheral ID
      AT91C_AIC_PRIOR_LOWEST, // Min priority
      AT91C_AIC_SRCTYPE_INT_LEVEL_SENSITIVE, // Level sensitive
      __sc_asm_interrupt_handler);
  }

void sc_setupUHP()
  {
    AT91PS_PMC pPMC = AT91C_BASE_PMC;
    pPMC->PMC_SCER = AT91C_PMC_UHP;      // enable clock //
    pPMC->PMC_PCER = 1 << AT91C_PMC_UHP; // enable peripheral //
    pPMC->PMC_PCER = 0xffffffff;
  }


void scReset()
  {
    *AT91C_ST_WDMR = 1 | (1 << 16);
    while (1);
  }

extern void __switch_cpu_clock(unsigned int iPLLAConfig, unsigned int iPLLBConfig, unsigned int iMDIV, unsigned int iNewSDRAMRefreshTB);

void scSwitchCPUClocks(unsigned int iPLLAConfig, unsigned int iPLLBConfig, unsigned int iMDIV, unsigned int iNewSDRAMRefreshTB)
  {
    __switch_cpu_clock(iPLLAConfig, iPLLBConfig, iMDIV, iNewSDRAMRefreshTB);
  }

