/**
 * Copyright (c) 2005 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#ifndef __sys_core_h
  #define __sys_core_h

extern void scHangUp(const char*);
extern void sc_LowLevelInit();

/// \brief Initializes the Interrupt controller.
extern void scInitAIC(void);
/// \brief Initializes the timer interrupt.
extern void scInitTimer(void);
/// \brief Initializes the debug port.
extern void scInitDBGU(unsigned int iMCK);
/// \brief Initializes the CPU clocks.
extern void scInitClocks();
/// \brief Initializes the SDRAM.
extern void scInitSDRAM(unsigned int iMCK, unsigned int iSDRAMConfig, unsigned int bo32Bit);
/// \brief Setup SDRAM self refresh.
extern void scSDRAMSelfRefresh();

/// \brief Tries to probe known sdram devices and configures them.
extern unsigned int scProbeSDRAM(unsigned int iMCK);
/// \brief Returns the size of sdram.
extern unsigned int getSDRAMSize(void);

/// \brief System reboot. (Uses wdt timeout.)
extern void scReset(void);


/**
  \brief Turns off interrupts and jumps to address iStart.

  This procedure will be used to start a user application located in SDRAM.
  The following parameters are required in boot(..):
    \li iMemSize    - size of memory installed in bytes.
    \li iSwitchAddr - the address of scSwitchCPUClock(...) in SDRAM
  These values are passed as parameters to void bool([unsigned int iMemSize, [unsigned int iSwitchAddr]]);
  You may use or omit the given values.
*/
extern void callUser(unsigned int iMemSize, unsigned int iSwitchAddr, unsigned int iStart);

/// \brief Set a new mcu clock speed.
extern void scSwitchCPUClocks(unsigned int iPLLAConfig, unsigned int iPLLBConfig, unsigned int iMDIV, unsigned int iNewSDRAMRefreshTB);
/// \brief Initializes the mcu clock speed.
extern void __setup_cpu_clock(unsigned int iPLLAConfig, unsigned int iPLLBConfig, unsigned int iMDIV);

#endif // __sys_core_h
