/**
 * Copyright (c) 2005 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#include <at/include/at91rm9200-def.h>
#include <sc/sys_debug.h>
#include <sc/sys_time.h>

static char pHexSyms[] = "0123456789abcdef";

void scDbgPrintChar(char b)
  {
    while (!((*AT91C_DBGU_CSR) & AT91C_US_TXRDY));
      *AT91C_DBGU_THR = (b & 0x1FF);
  }

void scDbg(const char* s)
  {
    if (!(s)) return;
    while (*s != 0)
      {
        if (*s == '\n')
          {
             scDbgPrintChar('\r');
             scDbgPrintChar('\n');
             s++;
          }
        else
          scDbgPrintChar(*s++);
      }
  }

void scDbgPrintHex32(unsigned int iValue)
  {
    int i;
    for (i = 0; i < 8; i++)
      {
        char bNum = pHexSyms[(iValue >> (28-(i<<2))) & 15];
        scDbgPrintChar(bNum);
      }
  }

void scDbgPrintHex8(unsigned char iValue)
  {
    int i;
    for (i = 0; i < 2; i++)
      {
        char bNum = pHexSyms[(iValue >> (4-(i<<2))) & 15];
        scDbgPrintChar(bNum);
      }
  }

/**
  Liefert das Byte zurck, welches von der Debug-Schnittstelle gelesen wurde.
  Der Rckgabewert -1 besagt, dass in der bergebenen Zeit kein Zeichen gelesen
  werden konnte. Der TimeOut-Wert 0 bedeutet endloses warten.
*/
int scDbgReadChar(unsigned int iTimeOutInMS)
  {
    unsigned int iTicks = sc_getTickCount();
    while ((sc_getTickCount() - iTicks < iTimeOutInMS*33) || (iTimeOutInMS == 0))
      {
        if (((*AT91C_DBGU_CSR) & AT91C_US_RXRDY) == 1)
          return ((*AT91C_DBGU_RHR) & 0x1FF);
      }
    return (-1);
  }

/**
  
*/
int scDbgHasChar()
  {
    return ((((*AT91C_DBGU_CSR) & AT91C_US_RXRDY) == 1));
  }

// -- string operations -- //

/*void* memcpy(void* _pTo, const void* _pFrom, unsigned int iCount)
  {
    int i;
    char* pTo = (char*)_pTo;
    const char* pFrom = (const char*)_pFrom;
    for (i = 0; i < iCount; i++)
      pTo[i] = pFrom[i];
    return (pTo);
  }*/

