/**
 * Copyright (c) 2005, 2006 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#include <sys_core.h>
#include <sc_config.h>
#include <libsc/sys_core.h>
#include <asm/mcu-direct.h>
#include "scll.h"

/*
  Initializes the MCU as follows:
    Core-Clock: 160MHz
    Bus-Clock:  80MHz
*/


/*
  \brief sc_setup will be called by entry.S to initialize the system.
  The following functions must be defined by user:
    void scInitStage0(void) - interrupt controller is initialized
    void scInitStage1(void) - clock has been initialized
    void scInitStage2(void) - external sdram has been initialized
*/
void sc_setup(void)
  {
    // sc_cli(); // interrupts are disabled by default //
    // Init Interrupt Controller
    scInitAIC();
    scInitDBGU(48000000);

    // Setup Interrupts //
    scInitTimer();

    __setup_cpu_clock(PLLA_VALUE_FAST(8,64), PLLB_VALUE_USB(45,216), SC_MAIN_CLOCK_DIV2);
    scInitDBGU(80000000);

    // The low level counterpart of user software. (Runs with 48MHz.)
    scInitStage0();

    // try setup sdram after starting clocks...
    scProbeSDRAM(80000000);
  }
