/**
 * Copyright (c) 2005, 2006 Peer Georgi <peer@peer-ware.de>
 * License: GPL
 *
 * This program is free software; you may redistribute and/or modify it under
 * the terms of the GNU General Public License Version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY, without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for complete details.
*/


#include <sys_core.h>
#include <sc_config.h>
#include <libsc/sys_core.h>
#include <asm/mcu-direct.h>
#include "scll-lc.h"

/*
  Initializes the MCU as follows:
    BUS-Clock: 20MHz
    Core-Clock:
  This low level initialization scheme does not setup high mcu clocks.
  It uses the (rom-firmware) preconfigured 48MHz mcu clock, initializes
  the SDRAM, timer and debug port and continues.

  You may use this scheme on USB powered devices to keep the current draw
  below 100mA while startup.
*/

extern void __setup_cpu_clock(unsigned int iPLLAConfig, unsigned int iPLLBConfig, unsigned int iMCK);


/*
  \brief sc_setup will be called by entry.S to initialize the system.
  The following functions must be defined by user:
    void scInitStage0(void) - interrupt controller is initialized
    void scInitStage1(void) - clock has been initialized
    void scInitStage2(void) - external sdram has been initialized
*/
void sc_setup(void)
  {
    unsigned int iMCK = 26666666;

    // sc_cli(); // interrupts are disabled by default //
    // Init Interrupt Controller
    scInitAIC();
    scInitDBGU(48000000);

    // Setup Interrupts //
    scInitTimer();

    __setup_cpu_clock(PLLA_VALUE_SLOW(8,32), PLLB_VALUE_USB(45,216), SC_MAIN_CLOCK_DIV3);
    scInitDBGU(iMCK);

    // The low level counterpart of user software. (Runs with 20MHz.)
    scInitStage0();

    // try setup sdram after starting clocks...
    scProbeSDRAM(iMCK);
  }
