/*
 *  linux/drivers/video/carmeva_uc1611.c -- Low level UC1611 based console driver
 *
 *  (c) 2005 Peer Georgi <georgi@conitec.comu>
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License.  See the file COPYING in the main directory of this archive for
 *  more details.
 *
 *  Changelog:
 *    (06/22/05) start working
 *    (02/21/06) ported to 2.6.15
 */


/*
  You have to modify vt.c to use this as default console.
*/


#include <linux/types.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/tty.h>
#include <linux/console.h>
#include <linux/string.h>
#include <linux/kd.h>
#include <linux/slab.h>
#include <linux/vt_kern.h>
#include <linux/vt_buffer.h>
#include <linux/selection.h>
#include <linux/spinlock.h>
#include <linux/ioport.h>
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/mm.h>

#include <linux/device.h>
#include <linux/platform_device.h>

#include <mach/hardware.h>
#include <mach/at91rm9200.h>
#include <mach/at91_tc.h>
#include <mach/at91_pmc.h>
#include <mach/at91rm9200_mc.h>


#include <mach/hardware.h>
#include <mach/at91rm9200.h>
#include <mach/at91_tc.h>
#include <mach/at91_pmc.h>
#include <mach/at91rm9200_mc.h>

#include <linux/module.h>
#include <mach/gpio.h>

//#include "f.h"
//#include "g.h"
#include "font_8x8.c"

#include "carmeva_uc1611.h"



//static DEFINE_SPINLOCK(carmeva_uc1611_lock);

static struct vc_data *carmeva_uc1611_display = NULL;


static ssize_t driverVersionRead(struct device_driver*, char*);
static ssize_t driverContrastSet(struct device_driver* pDriver, const char* p, size_t iSize);
static ssize_t driverBacklightSet(struct device_driver* pDriver, const char* p, size_t iSize);

static DRIVER_ATTR(version, S_IRUGO, &driverVersionRead, 0);
static DRIVER_ATTR(contrast, S_IWUGO, 0, &driverContrastSet);
static DRIVER_ATTR(backlight, S_IWUGO, 0, &driverBacklightSet);

static struct device_driver uc1611Driver =
  {
    .name = "lcd-uc1611",
    .bus = &platform_bus_type
  };


static struct tCArmEvaLCD gCArmEvaDisplay = {0,0,0,0, 0,0,0,0};

#define true (1==1)
#define false (1!=1)

#ifdef __CARMEVA_LCD_DEBUG
  #define POST_DEBUG(x) do { printk("%s",x); } while (0)
#else
  #define POST_DEBUG(x) do { } while (0)
#endif

// #include "carmeva_uc1611_io.c"


#ifndef MODULE
static int __init carmeva_uc1611_setup(char *str)
{
  /* command line format: carmeva_uc1611=<c>,<r> */

  int ints[3];

  str = get_options(str, ARRAY_SIZE(ints), ints);

  if (ints[0] < 2)
    return 0;

  if (ints[1] < 1 || ints[1] > MAX_NR_CONSOLES ||
      ints[2] < 1 || ints[2] > MAX_NR_CONSOLES)
    return 0;

//  mda_first_vc = ints[1];
//  mda_last_vc  = ints[2];
  return 1;
}

__setup("carmeva_uc1611=", carmeva_uc1611_setup);
#endif

static const char* __init carmeva_uc1611_startup(void)
  {
    // setup gpio, backlight and bus-interface //
    carmeva_lcd_setup(&gCArmEvaDisplay);
    if (!(carmeva_lcd_detect(&gCArmEvaDisplay)))
      {
        printk("carmeva_uc1611: No display connected.\n");
        return (0);
      }

    carmeva_lcd_clear(&gCArmEvaDisplay);
    carmeva_lcd_putString(&gCArmEvaDisplay, 0,0, "Hello World", strlen("Hello World"), false, 15, 0);
    carmeva_lcd_repaintAll(&gCArmEvaDisplay);
    return "ARM&EVA LCD";
  }

static void carmeva_uc1611_console_init(struct vc_data *c, int init)
  {
    unsigned int iCols = 240 / gCArmEvaDisplay.miCharWidth;
    unsigned int iRows = 160 / gCArmEvaDisplay.miCharHeight;

#ifdef __CARMEVA_LCD_DEBUG
    printk("CarmEvaConsole: %d/%d\n", gCArmEvaDisplay.miCharWidth, gCArmEvaDisplay.miCharHeight);

    printk("carmeva_uc1611_console_init::Console of size: %dx%d\n", iCols, iRows);
#endif
    c->vc_complement_mask = 0x0800;  /* reverse video */
    c->vc_display_fg = &carmeva_uc1611_display;
    c->vc_hi_font_mask = 0;
    c->vc_video_erase_char = '.';
    if (init)
      {
        c->vc_can_do_color = 0;
        c->vc_cols = iCols;
        c->vc_rows = iRows;
      }
    else
      vc_resize(c, iCols, iRows);

    if (carmeva_uc1611_display == NULL)
      carmeva_uc1611_display = c;
  }

static void carmeva_uc1611_console_destroy(struct vc_data *c)
  {
    con_set_default_unimap(c);

    if (carmeva_uc1611_display == c)
      carmeva_uc1611_display = 0;
  }

static u8 carmeva_uc1611_console_build_attr(struct vc_data *c, u8 color, u8 intensity, u8 blink, u8 underline, u8 reverse)
  {
    return (15);
    return (((color & 7) | ((intensity & 3) << 2)) & 15);
  }

static void carmeva_uc1611_console_invert_region(struct vc_data *c, u16 *p, int count)
  {
    POST_DEBUG("CarmEvaConsole InvertRegion not supported yet.\n");
//   for (; count > 0; count--) {
//     scr_writew(scr_readw(p) ^ 0x0800, p);
//     p++;
//   }
  }

static void carmeva_uc1611_console_putc(struct vc_data *c, int ch, int y, int x)
  {
    POST_DEBUG("CarmEvaConsole: PutC\n");
//  scr_writew(mda_convert_attr(ch), MDA_ADDR(x, y));
  }

static void carmeva_uc1611_console_putcs(struct vc_data *c, const unsigned short *s, int count, int y, int x)
  {
    unsigned int iX = x*(gCArmEvaDisplay.miCharWidth);
    unsigned int iY = y*(gCArmEvaDisplay.miCharHeight);
//    POST_DEBUG("PutCS");
#ifdef __CARMEVA_LCD_DEBUG
    int i;
    printk("iX:%d iY:%d %d :", iX, iY, count);
    for (i = 0; i < count; i++)
      printk("%c", s[i]);
    printk("\n");
#endif
    carmeva_lcd_putAttrString(&gCArmEvaDisplay, iX, iY, s, count, false, 15, 0);
    carmeva_lcd_repaintAll(&gCArmEvaDisplay);
  }

static void carmeva_uc1611_console_clear(struct vc_data *c, int y, int x, int height, int width)
  {
    int iY = 0;
    int iNumBytesToErase = width*gCArmEvaDisplay.miCharWidth;
    unsigned int iBytesOfLine = 240;
    if (y <= 0) return;
#ifdef __CARMEVA_LCD_DEBUG
    printk("carmeva_uc1611_console_clear %d %d %d %d\n", x, y, width, height);
#endif
    for (iY = (y*gCArmEvaDisplay.miCharHeight); iY < (((height+1)*gCArmEvaDisplay.miCharHeight)); iY++)
      {
        char* pDst = gCArmEvaDisplay.mpFB+(iY*iBytesOfLine)+(x*gCArmEvaDisplay.miCharWidth);
        memset(pDst, 0, iNumBytesToErase);
      }
//    carmeva_lcd_clear(&gCArmEvaDisplay);
    carmeva_lcd_repaintAll(&gCArmEvaDisplay);
  }

static void carmeva_uc1611_console_bmove(struct vc_data *c, int sy, int sx, int dy, int dx, int height, int width)
  {
    int iY = 0;
    unsigned int iBytesOfLine = 240;
    unsigned int iNumLinesPerRow = (gCArmEvaDisplay.miCharHeight);
    unsigned int iNumBytesPerRow = iNumLinesPerRow*iBytesOfLine;
#ifdef __CARMEVA_LCD_DEBUG
    printk("carmeva_uc1611_console_bmove %d %d %d %d: %dx%d\n", sx, sy, dx, dy, width, height);
#endif
    for (iY = 0; iY < ((height*gCArmEvaDisplay.miCharHeight)); iY++)
      {
        char* pSrc = 0;
        char* pDst = 0;
#ifdef __CARMEVA_LCD_DEBUG
        printk("Moving line iY:%d (srcY=%d;  dstY=%d)\n", iY, sy+iY, dy+iY);
#endif
        pSrc = gCArmEvaDisplay.mpFB+((sy+iY)*iNumBytesPerRow)+(sx*gCArmEvaDisplay.miCharWidth);
        pDst = gCArmEvaDisplay.mpFB+((dy+iY)*iNumBytesPerRow)+(dx*gCArmEvaDisplay.miCharWidth);
        memmove(pDst, pSrc, width*gCArmEvaDisplay.miCharWidth);
      }
  }

static int carmeva_uc1611_console_switch(struct vc_data *c)
  {
    return 1; /* redrawing needed */
  }

static int carmeva_uc1611_console_set_palette(struct vc_data *c, unsigned char *table)
  {
    return -EINVAL;
  }

static int carmeva_uc1611_console_blank(struct vc_data *c, int blank, int mode_switch)
  {
#ifdef __CARMEVA_LCD_DEBUG
    printk("carmeva_uc1611_console_blank - not supported yet\n");
#endif

//   if (mda_type == TYPE_MDA) {
//     if (blank)
//       scr_memsetw((void *)mda_vram_base,
//         mda_convert_attr(c->vc_video_erase_char),
//         c->vc_screenbuf_size);
//     /* Tell console.c that it has to restore the screen itself */
//     return 1;
//   } else {
//     if (blank)
//       outb_p(0x00, mda_mode_port);  /* disable video */
//     else
//       outb_p(MDA_MODE_VIDEO_EN | MDA_MODE_BLINK_EN,
//         mda_mode_port);
//     return 0;
//   }
    return (0);
  }

static int carmeva_uc1611_console_scrolldelta(struct vc_data *c, int lines)
  {
#ifdef __CARMEVA_LCD_DEBUG
    printk("carmeva_uc1611_console_scrolldelta - not supported yet\n");
#endif
    return 0;
  }

static void carmeva_uc1611_console_cursor(struct vc_data *c, int mode)
  {
//   if (mode == CM_ERASE) {
//     mda_set_cursor(mda_vram_len - 1);
//     return;
//   }
//
//   mda_set_cursor(c->vc_y*mda_num_columns*2 + c->vc_x*2);
//
//   switch (c->vc_cursor_type & 0x0f) {
//
//     case CUR_LOWER_THIRD: mda_set_cursor_size(10, 13); break;
//     case CUR_LOWER_HALF:  mda_set_cursor_size(7,  13); break;
//     case CUR_TWO_THIRDS:  mda_set_cursor_size(4,  13); break;
//     case CUR_BLOCK:   mda_set_cursor_size(1,  13); break;
//     case CUR_NONE:    mda_set_cursor_size(14, 13); break;
//     default:    mda_set_cursor_size(12, 13); break;
//   }
  }

static int carmeva_uc1611_console_scroll(struct vc_data *c, int t, int b, int dir, int iRows)
  {
    if (!(iRows)) return (0);
    if (iRows > c->vc_rows)
      iRows = c->vc_rows;
#ifdef __CARMEVA_LCD_DEBUG
    printk("carmeva_uc1611_console_scroll %d\n", dir);
#endif
    switch (dir)
      {
        case SM_UP:
          {
            unsigned int iBytesOfLine = 240;
            unsigned int iNumLinesPerRow = (gCArmEvaDisplay.miCharHeight);
            unsigned int iNumBytesPerRow = iNumLinesPerRow*iBytesOfLine;
            char* pSrc = 0;
            char* pDst = 0;

#ifdef __CARMEVA_LCD_DEBUG
            printk("CarmEvaConsole: ScrlUp\n");
#endif
            pSrc = gCArmEvaDisplay.mpFB+(iRows*iNumBytesPerRow);
            pDst = gCArmEvaDisplay.mpFB;
            memmove(pDst, pSrc, iNumBytesPerRow*c->vc_rows);

            /* fill the rest */
            memset(gCArmEvaDisplay.mpFB+((c->vc_rows)*iNumBytesPerRow)-(iRows*iNumBytesPerRow), 0, iNumBytesPerRow);
          }
         break;

        case SM_DOWN:
          {
            printk("CarmEvaConsole: ScrlDn not supported!\n");
          }
         break;
      }
    return (0);
//   u16 eattr = mda_convert_attr(c->vc_video_erase_char);
//
//   if (!lines)
//     return 0;
//
//   if (lines > c->vc_rows)   /* maximum realistic size */
//     lines = c->vc_rows;
//
//   switch (dir) {
//
//   case SM_UP:
//     scr_memmovew(MDA_ADDR(0,t), MDA_ADDR(0,t+lines),
//         (b-t-lines)*mda_num_columns*2);
//     scr_memsetw(MDA_ADDR(0,b-lines), eattr,
//         lines*mda_num_columns*2);
//     break;
//
//   case SM_DOWN:
//     scr_memmovew(MDA_ADDR(0,t+lines), MDA_ADDR(0,t),
//         (b-t-lines)*mda_num_columns*2);
//     scr_memsetw(MDA_ADDR(0,t), eattr, lines*mda_num_columns*2);
//     break;
//   }
    return 0;
  }

static ssize_t driverVersionRead(struct device_driver* pDriver, char* pBf)
  {
    snprintf(pBf, 256, "%d.%d\n", 0, 9);
    return (strlen(pBf)+1);
  }

static ssize_t driverContrastSet(struct device_driver* pDriver, const char* pBuffer, size_t iSize)
  {
    int iValue = 0;
    if (pBuffer)
      {
        if (sscanf(pBuffer, "%d", &iValue) == 1)
          {
            if (iValue < 0)
              iValue = 0;
            if (iValue > 255)
              iValue = 255;
            carmeva_lcd_set_contrast(&gCArmEvaDisplay, iValue);
          }
      }

    return (iSize);
  }

static ssize_t driverBacklightSet(struct device_driver* pDriver, const char* pBuffer, size_t iSize)
  {
    int iValue = 0;
    if (pBuffer)
      {
        if (sscanf(pBuffer, "%d", &iValue) == 1)
          {
            if (iValue < 0)
              iValue = 0;
            if (iValue > 100)
              iValue = 100;
            carmeva_lcd_set_backlight(&gCArmEvaDisplay, iValue);
          }
      }
    return (iSize);
  }


// PNM support //

#define DRIVER_MAJOR 240
/*
  device definitions
*/
static int drvPNMDeviceOpen(struct inode* pNode, struct file* pFile)
  {
    return (0);
  }

static int drvPNMDeviceClose(struct inode* pNode, struct file* pFile)
  {
    return (0);
  }

static int drvPNMDeviceWrite(struct file* pFile, const char* pBuffer, size_t iSize, loff_t* pPos)
  {
    if (*pPos != 0)
      return (-EINVAL);
//    printk("drawPNM of size %d\n", iSize);
    if (drawPNM(&gCArmEvaDisplay, 0,0, pBuffer, iSize))
      {
        carmeva_lcd_repaintAll(&gCArmEvaDisplay);
        return (iSize);
      }
    return (-EINVAL);
  }

static struct file_operations driverPNMFops =
  {
    .owner = THIS_MODULE,
    .ioctl =   0,
    .open =    drvPNMDeviceOpen,
    .release = drvPNMDeviceClose,
    .write =   drvPNMDeviceWrite,
    .read =    0
  };


static const struct consw carmeva_uc1611_console = {
  .owner =    THIS_MODULE,
  .con_startup =    carmeva_uc1611_startup,
  .con_init =   carmeva_uc1611_console_init,
  .con_deinit =   carmeva_uc1611_console_destroy,
  .con_clear =    carmeva_uc1611_console_clear,
  .con_putc =   carmeva_uc1611_console_putc,
  .con_putcs =    carmeva_uc1611_console_putcs,
  .con_cursor =   carmeva_uc1611_console_cursor,
  .con_scroll =   carmeva_uc1611_console_scroll,
  .con_bmove =    carmeva_uc1611_console_bmove,
  .con_switch =   carmeva_uc1611_console_switch,
  .con_blank =    carmeva_uc1611_console_blank,
  .con_set_palette =  carmeva_uc1611_console_set_palette,
  .con_scrolldelta =  carmeva_uc1611_console_scrolldelta,
  .con_build_attr = carmeva_uc1611_console_build_attr,
  .con_invert_region =  carmeva_uc1611_console_invert_region,
};

int __init carmeva_uc1611_init(void)
  {
    int iResult = 0;
    if (!(driver_register(&uc1611Driver)) == 0)
      return (-ENOMEM);

#ifdef __CARMEVA_LCD_DEBUG
    const char* sText = "Hello World!";
    printk("CAREVA_LCD Init!\n");
#endif
    gCArmEvaDisplay.mboMirrorX = false;
    gCArmEvaDisplay.mboMirrorY = true;
    gCArmEvaDisplay.mpIO = (unsigned char*)__arm_ioremap((unsigned long)0x80000000, L_PTE_PRESENT | L_PTE_YOUNG | L_PTE_DIRTY | L_PTE_WRITE , 1);
    if (gCArmEvaDisplay.mpIO)
      {
        gCArmEvaDisplay.mpFB = kmalloc(240*160, GFP_ATOMIC);
        if (!(gCArmEvaDisplay.mpFB))
          {
            __iounmap(gCArmEvaDisplay.mpIO);
            gCArmEvaDisplay.mpIO = 0;
          }
      }

   gCArmEvaDisplay.mpTCBBase = (unsigned char*)__arm_ioremap((unsigned long)AT91RM9200_BASE_TCB0, L_PTE_PRESENT | L_PTE_YOUNG | L_PTE_DIRTY | L_PTE_WRITE, 1);

#if 0
     gCArmEvaDisplay.mpFont = fontdata_mini_4x6;
     gCArmEvaDisplay.miFontMax = 256;
     gCArmEvaDisplay.miCharHeight = 6;
     gCArmEvaDisplay.miCharWidth = 4;
#else
    gCArmEvaDisplay.mpFont = fontdata_8x8;
    gCArmEvaDisplay.miFontMax = 256;
    gCArmEvaDisplay.miCharHeight = 8;
    gCArmEvaDisplay.miCharWidth = 8;
#endif

    carmeva_lcd_setup(&gCArmEvaDisplay);
#ifdef __CARMEVA_LCD_DEBUG
    printk("CarmEvaConsole: %d/%d\n", gCArmEvaDisplay.miCharWidth, gCArmEvaDisplay.miCharHeight);
#endif
    iResult = take_over_console(&carmeva_uc1611_console, 0, MAX_NR_CONSOLES-1, 1);
    if (iResult < 0)
      return (iResult);

    carmeva_lcd_clear(&gCArmEvaDisplay);
#ifdef __CARMEVA_LCD_DEBUG
    carmeva_lcd_putString(&gCArmEvaDisplay, 0,0, sText, strlen(sText), false, 15, 0);
#endif
    carmeva_lcd_repaintAll(&gCArmEvaDisplay);


    driver_create_file(&uc1611Driver, &driver_attr_version);
    driver_create_file(&uc1611Driver, &driver_attr_contrast);
    driver_create_file(&uc1611Driver, &driver_attr_backlight);

    register_chrdev(DRIVER_MAJOR, "uc1611-lcd", &driverPNMFops);

    return (iResult);
  }

static void __exit carmeva_uc1611_exit(void)
  {
    driver_remove_file(&uc1611Driver, &driver_attr_version);
    driver_remove_file(&uc1611Driver, &driver_attr_contrast);
    driver_remove_file(&uc1611Driver, &driver_attr_backlight);
    driver_unregister(&uc1611Driver);
    unregister_chrdev(DRIVER_MAJOR, "uc1611-lcd");

    give_up_console(&carmeva_uc1611_console);

    if (gCArmEvaDisplay.mpIO)
      __iounmap(gCArmEvaDisplay.mpIO);
    gCArmEvaDisplay.mpIO = 0;

    if (gCArmEvaDisplay.mpTCBBase)
      __iounmap(gCArmEvaDisplay.mpTCBBase);

    if (gCArmEvaDisplay.mpFB)
      kfree(gCArmEvaDisplay.mpFB);
    gCArmEvaDisplay.mpFB = 0;
  }

module_init(carmeva_uc1611_init);
module_exit(carmeva_uc1611_exit);

MODULE_AUTHOR("Peer Georgi");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Display driver for Arm&Eva display (UC1611 based).");
