
#include <mach/hardware.h>
#include <mach/at91rm9200.h>
#include <mach/at91_tc.h>
#include <mach/at91_pmc.h>
#include <mach/at91rm9200_mc.h>

#include <linux/module.h>
#include <mach/gpio.h>

#include "carmeva_uc1611.h"

// #define AT91_VA_BASE_TCB0       AT91_IO_P2V(AT91RM9200_BASE_TCB0)
// #define AT91_VA_BASE_TCB1       AT91_IO_P2V(AT91RM9200_BASE_TCB1)


enum CARMEVA_LCD_ACCESS_MODE { amFb, amCtrl };
typedef enum CARMEVA_LCD_ACCESS_MODE tCArmEvaLCDAM;


static inline unsigned long at91_tc_read(void __iomem* pBase, unsigned int iReg)
  {
//     void __iomem* pBase = (void __iomem*)AT91_VA_BASE_TCB0;
    return (__raw_readl(pBase+iReg));
    return 0;
  }

static inline unsigned long at91_tc_write(void __iomem* pBase, unsigned int iReg, unsigned long iValue)
  {
//     void __iomem* pBase = (void __iomem*)AT91_VA_BASE_TCB0;
    return (__raw_writel(iValue, pBase+iReg));
    return 0;
  }

/**
  Set the percental backlight level (0..100%).
*/
void carmeva_lcd_set_backlight(struct tCArmEvaLCD* p, unsigned int b)
  {
    if (!p->mpTCBBase) return;

    unsigned int iValue = ((65535*(100-b)) / 100);
    if (b > 100) b = 0;
    if (iValue < 1)
      iValue = 1;

    at91_tc_write(p->mpTCBBase, AT91_TC_CMR, AT91_TC_ASWTRG_CLEAR | AT91_TC_WAVE | AT91_TC_ACPA_SET | AT91_TC_ACPC_CLEAR);
    at91_tc_write(p->mpTCBBase, AT91_TC_RA, iValue);
    at91_tc_write(p->mpTCBBase, AT91_TC_RC, 0xffff);
    at91_tc_write(p->mpTCBBase, AT91_TC_CCR, AT91_TC_CLKEN);
    at91_tc_write(p->mpTCBBase, AT91_TC_CCR, AT91_TC_SWTRG);
  }

/**
  This initializes I/O-ports assigned with the LCD.
  To allow setting a backlight level the hardware pwm will be used.
*/
void carmeva_lcd_setup(struct tCArmEvaLCD* p)
  {
    at91_sys_write(AT91_PMC_PCER, (1 << AT91RM9200_ID_TC0));

    // setup backlight //
    at91_set_B_periph(AT91_PIN_PA17, 0);
    // setup pwm for backlight //

    if (p->mpTCBBase)
      {
        at91_tc_write(p->mpTCBBase, AT91_TC_CCR, 0);
        carmeva_lcd_set_backlight(p, 0);
      }

    at91_set_gpio_output(AT91_PIN_PA5, 1);

    at91_sys_write(AT91_SMC_CSR(7), (AT91_SMC_NWS & 0x01) | AT91_SMC_WSEN | AT91_SMC_DBW_8);
  }




void carmeva_lcd_setAccessMode(struct tCArmEvaLCD*p, tCArmEvaLCDAM am)
  {
//    AT91S_SYS* pAT91_SYS = (AT91S_SYS*)AT91C_VA_BASE_SYS;
    switch (am)
      {
      case amFb:
        at91_set_gpio_output(AT91_PIN_PA5, 1);
//        pAT91_SYS->PIOA_SODR = 1 << 5;
       break;
      case amCtrl:
//        pAT91_SYS->PIOA_CODR = 1 << 5;
        at91_set_gpio_output(AT91_PIN_PA5, 0);
       break;
      }
  }

#include "carmeva_uc1611.h"

char carmeva_lcd_readStatus(struct tCArmEvaLCD* p)
  {
    carmeva_lcd_setAccessMode(p, amCtrl);
    return (*p->mpIO);
  }

void carmeva_lcd_writeInstruction(struct tCArmEvaLCD* p, unsigned char b)
  {
    carmeva_lcd_setAccessMode(p, amCtrl);
    *p->mpIO = b;
  }

void carmeva_lcd_set_contrast(struct tCArmEvaLCD* p, unsigned int b)
  {
    int iBiasPot = ((b >> 2) & 63);
    int iBiasGain = (b & 3);
/*    int iBiasPot = (b & 63);
    int iBiasGain = (b >> 6) & 3;*/
    carmeva_lcd_writeInstruction(p, 0xe8 | 1); // Bias Ratio
    carmeva_lcd_writeInstruction(p, 0x2a); // panel loading //
    carmeva_lcd_writeInstruction(p, 0x81); // LCD-Gain ans Bias Pot //
//      writeInstruction(0x40); // LCD-BIAS Gain ( (Gain & 3) | ((BiasPot & 63) << 2)
    carmeva_lcd_writeInstruction(p, ((iBiasPot & 63) << 2) | (iBiasGain & 3)); // LCD-Gain ans Bias Pot //
  }

void carmeva_lcd_set_mapping(struct tCArmEvaLCD* p)
  {
    unsigned char bInstruction = 0xc0;
    int iBiasGain = 2;
    int iBiasPot = 0x11;

    if (p->mboMirrorX)
      bInstruction |= 2;
    if (p->mboMirrorY)
      bInstruction |= 4;
    carmeva_lcd_writeInstruction(p, 0xd2); // grayscale (16 colors) //

    carmeva_lcd_writeInstruction(p, bInstruction); // lcd-  mapping //
    carmeva_lcd_writeInstruction(p, 0xe8 | 2); // Bias Ratio
    carmeva_lcd_writeInstruction(p, 0x2a); // panel loading //
    carmeva_lcd_writeInstruction(p, 0x81); // LCD-Gain ans Bias Pot //
    carmeva_lcd_writeInstruction(p, ((iBiasPot & 63) << 2) | (iBiasGain & 3)); // LCD-Gain ans Bias Pot //


    carmeva_lcd_writeInstruction(p, 0xa1); // line rate //
    carmeva_lcd_writeInstruction(p, 0x84); // set partial display control ??? //
    carmeva_lcd_writeInstruction(p, 0x89); // ram address control //
    carmeva_lcd_writeInstruction(p, 0x31); // adv. control //
    carmeva_lcd_writeInstruction(p, 0x81); // ???? //
    carmeva_lcd_writeInstruction(p, 0xaf); // enable display //
    carmeva_lcd_writeInstruction(p, 0x50); // scroll //
    carmeva_lcd_writeInstruction(p, 0x40); // scroll //

    carmeva_lcd_writeInstruction(p, 0x60);
    carmeva_lcd_writeInstruction(p, 0x70);
    carmeva_lcd_writeInstruction(p, 0x0);
    carmeva_lcd_writeInstruction(p, 0x10);
  }

/**
  Returns (1==1) on success and (1!=1) on failure.
*/
unsigned int carmeva_lcd_detect(struct tCArmEvaLCD* p)
  {
    // try change the mirror settings //
    carmeva_lcd_writeInstruction(p, 0xc0); // lcd-  mapping //
    if ((carmeva_lcd_readStatus(p) & 0x60) != 0) return (false);
    carmeva_lcd_writeInstruction(p, 0xc6); // lcd-  mapping //
    if ((carmeva_lcd_readStatus(p) & 0x60) != 0x60) return (false);
    carmeva_lcd_writeInstruction(p, 0xc4); // lcd-  mapping //
    if ((carmeva_lcd_readStatus(p) & 0x60) != 0x20) return (false);
    carmeva_lcd_writeInstruction(p, 0xc6); // lcd-  mapping //
    if ((carmeva_lcd_readStatus(p) & 0x60) != 0x60) return (false);
    carmeva_lcd_writeInstruction(p, 0xc0); // lcd-  mapping //
    if ((carmeva_lcd_readStatus(p) & 0x60) != 0) return (false);
    carmeva_lcd_set_mapping(p); // restore values //
    return (true);
  }

void carmeva_lcd_repaintAll(struct tCArmEvaLCD* p)
  {
    volatile unsigned int* pDisplay32 = (unsigned int*)p->mpIO;
    unsigned int* pFB32 = (unsigned int*)p->mpFB;
    unsigned int iLineOffset = 0;
    unsigned int iLine = 0;
    unsigned int iCurrentLineSetX = 0;
    carmeva_lcd_writeInstruction(p, 0x60);
    carmeva_lcd_writeInstruction(p, 0x70);
    carmeva_lcd_writeInstruction(p, 0x00);
    carmeva_lcd_writeInstruction(p, 0x10);
    carmeva_lcd_setAccessMode(p, amFb);
    for (iLine = 0; iLine < ((160 >> 1)); iLine++)
      {
        // convert line-orientation //
        for (iCurrentLineSetX = 0; iCurrentLineSetX < 240 >> 2; iCurrentLineSetX++)
          {
            register unsigned int i = pFB32[iCurrentLineSetX+iLineOffset] | (pFB32[iCurrentLineSetX+(240>>2)+iLineOffset] << 4);
            *pDisplay32 = i;
          }
        iLineOffset += 120; // (240*2) / 4
      }
  }

void carmeva_lcd_clear(struct tCArmEvaLCD* p)
  {
    memset(p->mpFB, 0, 240*160);
  }

void carmeva_lcd_setPixel(struct tCArmEvaLCD* p, unsigned int iX, unsigned int iY, char bCol)
  {
    p->mpFB[((iY)*240)+iX] = bCol;
  }

unsigned int getCharLine(struct tCArmEvaLCD* p, int iAscii, unsigned int iLine)
  {
    return (p->mpFont[iLine+(p->miCharHeight*iAscii)]);
  }

void carmeva_lcd_putChar(struct tCArmEvaLCD* p, unsigned int iX, unsigned int iY, char iAscii, unsigned int boTransp, char bCol, char bBackground)
  {
    int iValue = 0;
    unsigned int ix, iy;
    unsigned int iHeight = p->miCharHeight;
    unsigned int iWidth = p->miCharWidth;


    if (iAscii > p->miFontMax) return;
    if (iAscii == 0) return;

    for (iy=0; iy < iHeight; iy++)
      for (ix=0; ix < iWidth; ix++)
        {
          iValue = getCharLine(p, iAscii, iy);
          if (iValue & (1 << ((((iWidth-1)-ix)))))
            p->mpFB[iX+ix + ((iY+iy)*240)] = bCol;
          else
            if (!(boTransp))
             p->mpFB[iX+ix + ((iY+iy)*240)] = bBackground;
        }
  }

void carmeva_lcd_putString(struct tCArmEvaLCD* p, unsigned int iX, unsigned int iY, const char* sText, unsigned int iCount, unsigned int boTransp, char bCol, char bBackground)
   {
     char s = *sText;
     while (iCount--)
       {
         carmeva_lcd_putChar(p, iX, iY, s, boTransp, bCol, bBackground);
         iX += p->miCharWidth;
         s = *++sText;
       }
   }

void carmeva_lcd_putAttrString(struct tCArmEvaLCD* p, unsigned int iX, unsigned int iY, const unsigned short* sText, unsigned int iCount, unsigned int boTransp, char bCol, char bBackground)
   {
     short i = *sText;
     while (iCount--)
       {
         unsigned int iAttr = (i >> 8) & 255;
         bCol = iAttr;
         bBackground = 0;
         carmeva_lcd_putChar(p, iX, iY, i & 255, boTransp, bCol, bBackground);
         iX += p->miCharWidth;
         i = *++sText;
       }
   }


struct tPNMHandle
  {
    const char* mpPNMData;
    unsigned int miIndex;
    unsigned int miMaxSize;
  };

// << PNM extension >> //
/**
  The mpPNMData points to the first char of line-
*/
bool readPNMLine(struct tPNMHandle* pHandle)
  {
    while (pHandle->miIndex < pHandle->miMaxSize)
      {
        char b = pHandle->mpPNMData[pHandle->miIndex];
        if ((b != '\r') && (b != '\n') && (b != 0))
          return(TRUE);
        pHandle->miIndex++;
      }
    return (FALSE);
  }


unsigned int readNumber(struct tPNMHandle* pHandle, bool* piOk)
  {
    // read width //
    unsigned int iReturn = 0;
    unsigned int iDigit = 0;
    *piOk = FALSE;
    while (pHandle->miIndex < pHandle->miMaxSize)
      {
        char s = pHandle->mpPNMData[pHandle->miIndex++];
        if (s == 0)
          break;
        if (s == '\r')
          break;
        if (s == '\n')
          break;
        if (s == 32)
          if (iDigit)
            break;
        if ((s < '0') || (s > '9'))
          return (0);
        iReturn *= 10;
        iReturn += s-'0';
        iDigit++;
      }
    *piOk = TRUE;
    return (iReturn);
  }

bool pnmStrCmp(struct tPNMHandle* pHandle, const char* pS)
  {
    int iSizeLeft = pHandle->miMaxSize-strlen(pS);
    if (iSizeLeft < 0)
      return (false);
    if (strncmp(pHandle->mpPNMData, pS, strlen(pS)) == 0)
      {
        pHandle->miIndex += strlen(pS);
        return (true);
      }
    return (false);
  }


bool drawPNM(struct tCArmEvaLCD* pDisplay, unsigned int iX, unsigned int iY, const char* pPNMAddr, unsigned int iSize)
  {
    struct tPNMHandle pnmHandle;
    bool boGrayScale = false;

    pnmHandle.miIndex = 0;
    pnmHandle.mpPNMData = pPNMAddr;
    pnmHandle.miMaxSize = iSize;

    readPNMLine(&pnmHandle);
    if (pnmStrCmp(&pnmHandle, "P5"))
      boGrayScale = TRUE;

    if (!(boGrayScale))
      if (!(pnmStrCmp(&pnmHandle, "P6")))
        return (FALSE);

    readPNMLine(&pnmHandle);

    {
      bool boOk = false;
      unsigned int iWidth = readNumber(&pnmHandle, &boOk);
      unsigned int iHeight = readNumber(&pnmHandle, &boOk);
      if (!(iWidth))
        return (FALSE); // invalid width //
      if (!(iHeight))
        return (FALSE); // invalid height //

    readPNMLine(&pnmHandle);

      {
        unsigned int iMaxColor = readNumber(&pnmHandle, &boOk);
        if (iMaxColor != 255)
          return (FALSE);

        if (boGrayScale)
          {
            unsigned int ix = 0;
            unsigned int iy = 0;
            if (pnmHandle.miIndex+(iWidth*iHeight) > pnmHandle.miMaxSize)
              {
                printk("Data too small (2)\n");
                return (FALSE);
              }

            for (iy = 0; iy < iHeight; iy++)
              for (ix = 0; ix < iWidth; ix++)
                {
                  unsigned int bGs = pnmHandle.mpPNMData[pnmHandle.miIndex++];
                  carmeva_lcd_setPixel(pDisplay, iX+ix, iY+iy, 15-(bGs >> 4));
                }
          }
        else
          {
            unsigned int ix = 0;
            unsigned int iy = 0;
            if (pnmHandle.miIndex+(iWidth*iHeight)*3 > pnmHandle.miMaxSize)
              {
                 printk("Data too small (2)\n");
                 return (FALSE);
              }
            for (iy = 0; iy < iHeight; iy++)
              for (ix = 0; ix < iWidth; ix++)
                {
                  unsigned int bR = pnmHandle.mpPNMData[pnmHandle.miIndex++];
                  unsigned int bG = pnmHandle.mpPNMData[pnmHandle.miIndex++];
                  unsigned int bB = pnmHandle.mpPNMData[pnmHandle.miIndex++];
                  carmeva_lcd_setPixel(pDisplay, iX+ix, iY+iy, 15-(((bR+bG+bB) / 3) >> 4));
                }
          }
      }
   }
    return (true);
  }
